# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ServersNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of servers"""

    create_default_network = True

    def setUp(self):
        super(ServersNegativeTestJSON, self).setUp()
        try:
            waiters.wait_for_server_status(self.client, self.server_id,
                                           'ACTIVE')
        except Exception:
            self.__class__.server_id = self.recreate_server(self.server_id)

    def tearDown(self):
        super(ServersNegativeTestJSON, self).tearDown()
        # NOTE(zhufl): Because server_check_teardown will raise Exception
        # which will prevent other cleanup steps from being executed, so
        # server_check_teardown should be called after super's tearDown.
        self.server_check_teardown()

    @classmethod
    def setup_clients(cls):
        super(ServersNegativeTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServersNegativeTestJSON, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        cls.server_id = server['id']

        # Wait until the instance is active to avoid the delete racing
        server = cls.create_test_server(wait_until='ACTIVE')
        cls.client.delete_server(server['id'])
        waiters.wait_for_server_termination(cls.client, server['id'])
        cls.deleted_server_id = server['id']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('dbbfd247-c40c-449e-8f6c-d2aa7c7da7cf')
    def test_server_name_blank(self):
        """Creating a server with name parameter empty should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b8a7235e-5246-4a8f-a08e-b34877c6586f')
    @testtools.skipUnless(CONF.compute_feature_enabled.personality,
                          'Nova personality feature disabled')
    def test_personality_file_contents_not_encoded(self):
        """Using an unencoded injected file to create server should fail"""
        file_contents = 'This is a test file.'
        person = [{'path': '/etc/testfile.txt',
                   'contents': file_contents}]

        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          personality=person)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fcba1052-0a50-4cf3-b1ac-fae241edf02f')
    def test_create_with_invalid_image(self):
        """Creating a server with an unknown image should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          image_id=-1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('18f5227f-d155-4429-807c-ccb103887537')
    def test_create_with_invalid_flavor(self):
        """Creating a server with an unknown flavor should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          flavor=-1,)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7f70a4d1-608f-4794-9e56-cb182765972c')
    def test_invalid_access_ip_v4_address(self):
        """Creating a server with invalid ipv4 ip address should fail

        An access IPv4 address must match a valid address pattern
        """
        IPv4 = '1.1.1.1.1.1'
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server, accessIPv4=IPv4)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5226dd80-1e9c-4d8a-b5f9-b26ca4763fd0')
    def test_invalid_ip_v6_address(self):
        """Creating a server with invalid ipv6 ip address should fail

        An access IPv6 address must match a valid address pattern
        """
        IPv6 = 'notvalid'

        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server, accessIPv6=IPv6)

    @decorators.idempotent_id('7ea45b3e-e770-46fa-bfcc-9daaf6d987c0')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @decorators.attr(type=['negative'])
    def test_resize_nonexistent_server(self):
        """Resizing a non-existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.resize_server,
                          nonexistent_server, self.flavor_ref)

    @decorators.idempotent_id('ced1a1d7-2ab6-45c9-b90f-b27d87b30efd')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @decorators.attr(type=['negative'])
    def test_resize_server_with_non_existent_flavor(self):
        """Resizing a server with non existent flavor should fail"""
        nonexistent_flavor = data_utils.rand_uuid()
        self.assertRaises(lib_exc.BadRequest, self.client.resize_server,
                          self.server_id, flavor_ref=nonexistent_flavor)

    @decorators.idempotent_id('45436a7d-a388-4a35-a9d8-3adc5d0d940b')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @decorators.attr(type=['negative'])
    def test_resize_server_with_null_flavor(self):
        """Resizing a server with null flavor should fail"""
        self.assertRaises(lib_exc.BadRequest, self.client.resize_server,
                          self.server_id, flavor_ref="")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d4c023a0-9c55-4747-9dd5-413b820143c7')
    def test_reboot_non_existent_server(self):
        """Rebooting a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.reboot_server,
                          nonexistent_server, type='SOFT')

    @decorators.idempotent_id('d1417e7f-a509-41b5-a102-d5eed8613369')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type=['negative'])
    def test_pause_paused_server(self):
        """Pausing a paused server should fail"""
        self.client.pause_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id, 'PAUSED')
        self.assertRaises(lib_exc.Conflict,
                          self.client.pause_server,
                          self.server_id)
        self.client.unpause_server(self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('98fa0458-1485-440f-873b-fe7f0d714930')
    def test_rebuild_deleted_server(self):
        """Rebuilding a deleted server should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.rebuild_server,
                          self.deleted_server_id, self.image_ref)

    @decorators.related_bug('1660878', status_code=409)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('581a397d-5eab-486f-9cf9-1014bbd4c984')
    def test_reboot_deleted_server(self):
        """Rebooting a deleted server should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.reboot_server,
                          self.deleted_server_id, type='SOFT')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d86141a7-906e-4731-b187-d64a2ea61422')
    def test_rebuild_non_existent_server(self):
        """Rebuilding a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.rebuild_server,
                          nonexistent_server,
                          self.image_ref)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fd57f159-68d6-4c2a-902b-03070828a87e')
    def test_create_numeric_server_name(self):
        """Creating a server with numeric server name should fail"""
        server_name = 12345
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          name=server_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c3e0fb12-07fc-4d76-a22e-37409887afe8')
    def test_create_server_name_length_exceeds_256(self):
        """Creating a server with name length exceeding limit should fail

        Create a server with name length exceeding 255 characters, an error is
        returned.
        """
        server_name = 'a' * 256
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          name=server_name)

    @decorators.attr(type=['negative'])
    @decorators.related_bug('1651064', status_code=500)
    @utils.services('volume')
    @decorators.idempotent_id('12146ac1-d7df-4928-ad25-b1f99e5286cd')
    def test_create_server_invalid_bdm_in_2nd_dict(self):
        """Creating a server with invalid block_device_mapping_v2 should fail

        Create a server with invalid block_device_mapping_v2, an error is
        returned.
        """
        volume = self.create_volume()
        bdm_1st = {"source_type": "image",
                   "delete_on_termination": True,
                   "boot_index": 0,
                   "uuid": self.image_ref,
                   "destination_type": "local"}
        bdm_2nd = {"source_type": "volume",
                   "uuid": volume["id"],
                   "destination_type": "invalid"}
        bdm = [bdm_1st, bdm_2nd]

        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          image_id=self.image_ref,
                          block_device_mapping_v2=bdm)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4e72dc2d-44c5-4336-9667-f7972e95c402')
    def test_create_with_invalid_network_uuid(self):
        """Creating a server with invalid network uuid should fail"""
        # Pass invalid network uuid while creating a server
        networks = [{'fixed_ip': '10.0.1.1', 'uuid': 'a-b-c-d-e-f-g-h-i-j'}]
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          networks=networks)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7a2efc39-530c-47de-b875-2dd01c8d39bd')
    def test_create_with_non_existent_keypair(self):
        """Creating a server with non-existent keypair should fail"""
        # Pass a non-existent keypair while creating a server
        key_name = data_utils.rand_name('key')
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          key_name=key_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7fc74810-0bd2-4cd7-8244-4f33a9db865a')
    def test_create_server_metadata_exceeds_length_limit(self):
        """Creating a server with metadata longer than limit should fail """
        # Pass really long metadata while creating a server
        metadata = {'a': 'b' * 260}
        self.assertRaises((lib_exc.BadRequest, lib_exc.OverLimit),
                          self.create_test_server,
                          metadata=metadata)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('aa8eed43-e2cb-4ebf-930b-da14f6a21d81')
    def test_update_name_of_non_existent_server(self):
        """Updating name of a non-existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        new_name = data_utils.rand_name(
            self.__class__.__name__ + '-server') + '_updated'

        self.assertRaises(lib_exc.NotFound, self.client.update_server,
                          nonexistent_server, name=new_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('38204696-17c6-44da-9590-40f87fb5a899')
    def test_update_server_set_empty_name(self):
        """Updating name of the server to an empty string should fail"""
        new_name = ''
        self.assertRaises(lib_exc.BadRequest, self.client.update_server,
                          self.server_id, name=new_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5c8e244c-dada-4590-9944-749c455b431f')
    def test_update_server_name_length_exceeds_256(self):
        """Updating name of server exceeding the name length limit should fail

        Update name of server exceeding the name length limit, an error is
        returned.
        """
        new_name = 'a' * 256
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_server,
                          self.server_id,
                          name=new_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1041b4e6-514b-4855-96a5-e974b60870a3')
    def test_delete_non_existent_server(self):
        """Deleting a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.delete_server,
                          nonexistent_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('75f79124-277c-45e6-a373-a1d6803f4cc4')
    def test_delete_server_pass_negative_id(self):
        """Passing an invalid string parameter to delete server should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_server, -1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f4d7279b-5fd2-4bf2-9ba4-ae35df0d18c5')
    def test_delete_server_pass_id_exceeding_length_limit(self):
        """Deleting server with a server ID exceeding length limit should fail

        Pass a server ID that exceeds length limit to delete server, an error
        is returned.
        """
        self.assertRaises(lib_exc.NotFound, self.client.delete_server,
                          sys.maxsize + 1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c5fa6041-80cd-483b-aa6d-4e45f19d093c')
    def test_create_with_nonexistent_security_group(self):
        """Creating a server with a nonexistent security group should fail"""
        security_groups = [{'name': 'does_not_exist'}]
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          security_groups=security_groups)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('3436b02f-1b1e-4f03-881e-c6a602327439')
    def test_get_non_existent_server(self):
        """Getting a non existent server details should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.show_server,
                          nonexistent_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a31460a9-49e1-42aa-82ee-06e0bb7c2d03')
    def test_stop_non_existent_server(self):
        """Stopping a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.servers_client.stop_server,
                          nonexistent_server)

    @decorators.idempotent_id('6a8dc0c6-6cd4-4c0a-9f32-413881828091')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type=['negative'])
    def test_pause_non_existent_server(self):
        """Pausing a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.pause_server,
                          nonexistent_server)

    @decorators.idempotent_id('705b8e3a-e8a7-477c-a19b-6868fc24ac75')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type=['negative'])
    def test_unpause_non_existent_server(self):
        """Unpausing a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.unpause_server,
                          nonexistent_server)

    @decorators.idempotent_id('c8e639a7-ece8-42dd-a2e0-49615917ba4f')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type=['negative'])
    def test_unpause_server_invalid_state(self):
        """Unpausing an active server should fail"""
        self.assertRaises(lib_exc.Conflict,
                          self.client.unpause_server,
                          self.server_id)

    @decorators.idempotent_id('d1f032d5-7b6e-48aa-b252-d5f16dd994ca')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type=['negative'])
    def test_suspend_non_existent_server(self):
        """Suspending a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.suspend_server,
                          nonexistent_server)

    @decorators.idempotent_id('7f323206-05a9-4bf8-996b-dd5b2036501b')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type=['negative'])
    def test_suspend_server_invalid_state(self):
        """Suspending a suspended server should fail"""
        self.client.suspend_server(self.server_id)
        waiters.wait_for_server_status(self.client, self.server_id,
                                       'SUSPENDED')
        self.assertRaises(lib_exc.Conflict,
                          self.client.suspend_server,
                          self.server_id)
        self.client.resume_server(self.server_id)

    @decorators.idempotent_id('221cd282-bddb-4837-a683-89c2487389b6')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type=['negative'])
    def test_resume_non_existent_server(self):
        """Resuming a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.resume_server,
                          nonexistent_server)

    @decorators.idempotent_id('ccb6294d-c4c9-498f-8a43-554c098bfadb')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type=['negative'])
    def test_resume_server_invalid_state(self):
        """Resuming an active server should fail"""
        self.assertRaises(lib_exc.Conflict,
                          self.client.resume_server,
                          self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7dd919e7-413f-4198-bebb-35e2a01b13e9')
    def test_get_console_output_of_non_existent_server(self):
        """Getting the console output for a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.get_console_output,
                          nonexistent_server, length=10)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6f47992b-5144-4250-9f8b-f00aa33950f3')
    def test_force_delete_nonexistent_server_id(self):
        """Force-deleting a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.force_delete_server,
                          nonexistent_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9c6d38cc-fcfb-437a-85b9-7b788af8bf01')
    def test_restore_nonexistent_server_id(self):
        """Restore-deleting a non existent server should fail

        We can restore a soft deleted server, but can't restore a non
        existent server.
        """
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.restore_soft_deleted_server,
                          nonexistent_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7fcadfab-bd6a-4753-8db7-4a51e51aade9')
    def test_restore_server_invalid_state(self):
        """Restore-deleting a server not in 'soft-delete' state should fail

        We can restore a soft deleted server, but can't restore a server that
        is not in 'soft-delete' state.
        """
        self.assertRaises(lib_exc.Conflict,
                          self.client.restore_soft_deleted_server,
                          self.server_id)

    @decorators.idempotent_id('abca56e2-a892-48ea-b5e5-e07e69774816')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @decorators.attr(type=['negative'])
    def test_shelve_non_existent_server(self):
        """Shelving a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.shelve_server,
                          nonexistent_server)

    @decorators.idempotent_id('443e4f9b-e6bf-4389-b601-3a710f15fddd')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @decorators.attr(type=['negative'])
    def test_shelve_shelved_server(self):
        """Shelving a shelved server should fail"""
        compute.shelve_server(self.client, self.server_id)

        def _unshelve_server():
            server_info = self.client.show_server(self.server_id)['server']
            if 'SHELVED' in server_info['status']:
                self.client.unshelve_server(self.server_id)
        self.addCleanup(_unshelve_server)

        server = self.client.show_server(self.server_id)['server']
        image_name = server['name'] + '-shelved'
        if CONF.image_feature_enabled.api_v1:
            kwargs = {'name': image_name}
        else:
            kwargs = {'params': {'name': image_name}}
        images = self.images_client.list_images(**kwargs)['images']
        self.assertEqual(1, len(images))
        self.assertEqual(image_name, images[0]['name'])

        self.assertRaises(lib_exc.Conflict,
                          self.client.shelve_server,
                          self.server_id)

        self.client.unshelve_server(self.server_id)

    @decorators.idempotent_id('23d23b37-afaf-40d7-aa5d-5726f82d8821')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @decorators.attr(type=['negative'])
    def test_unshelve_non_existent_server(self):
        """Unshelving a non existent server should fail"""
        nonexistent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.unshelve_server,
                          nonexistent_server)

    @decorators.idempotent_id('8f198ded-1cca-4228-9e65-c6b449c54880')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve,
                          'Shelve is not available.')
    @decorators.attr(type=['negative'])
    def test_unshelve_server_invalid_state(self):
        """Unshelving an active server should fail"""
        self.assertRaises(lib_exc.Conflict,
                          self.client.unshelve_server,
                          self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('74085be3-a370-4ca2-bc51-2d0e10e0f573')
    @utils.services('volume', 'image')
    def test_create_server_from_non_bootable_volume(self):
        """Creating a server from a non bootable volume should fail"""
        volume = self.create_volume()

        # Update volume bootable status to false
        self.volumes_client.set_bootable_volume(volume['id'],
                                                bootable=False)

        # Verify bootable flag was updated
        nonbootable_vol = self.volumes_client.show_volume(
            volume['id'])['volume']
        self.assertEqual('false', nonbootable_vol['bootable'])

        # Block device mapping
        bd_map = [{'boot_index': '0',
                   'uuid': volume['id'],
                   'source_type': 'volume',
                   'destination_type': 'volume',
                   'delete_on_termination': False}]

        # Try creating a server from non-bootable volume
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          image_id='',
                          wait_until='ACTIVE',
                          block_device_mapping_v2=bd_map)


class ServersNegativeTestMultiTenantJSON(base.BaseV2ComputeTest):
    """Negative tests of servers for multiple projects"""

    create_default_network = True

    credentials = ['primary', 'alt']

    def setUp(self):
        super(ServersNegativeTestMultiTenantJSON, self).setUp()
        try:
            waiters.wait_for_server_status(self.servers_client, self.server_id,
                                           'ACTIVE')
        except Exception:
            self.__class__.server_id = self.recreate_server(self.server_id)

    @classmethod
    def setup_clients(cls):
        super(ServersNegativeTestMultiTenantJSON, cls).setup_clients()
        cls.alt_client = cls.os_alt.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServersNegativeTestMultiTenantJSON, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        cls.server_id = server['id']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('543d84c1-dd2e-4c6d-8cb2-b9da0efaa384')
    def test_update_server_of_another_tenant(self):
        """Updating server that belongs to another project should fail

        Update name of a server that belongs to another project, an error is
        returned.
        """
        new_name = self.server_id + '_new'
        self.assertRaises(lib_exc.NotFound,
                          self.alt_client.update_server, self.server_id,
                          name=new_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5c75009d-3eea-423e-bea3-61b09fd25f9c')
    def test_delete_a_server_of_another_tenant(self):
        """Deleting a server that belongs to another project should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.alt_client.delete_server,
                          self.server_id)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import schemas_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSchemasClient(base.BaseServiceTest):
    FAKE_SHOW_SCHEMA = {
        "links": [
            {
                "href": "{schema}",
                "rel": "describedby"
            }
        ],
        "name": "members",
        "properties": {
            "members": {
                "items": {
                    "name": "member",
                    "properties": {
                        "created_at": {
                            "description": ("Date and time of image member"
                                            " creation"),
                            "type": "string"
                        },
                        "image_id": {
                            "description": "An identifier for the image",
                            "pattern": ("^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}"
                                        "-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}"
                                        "-([0-9a-fA-F]){12}$"),
                            "type": "string"
                        },
                        "member_id": {
                            "description": ("An identifier for the image"
                                            " member (tenantId)"),
                            "type": "string"
                        },
                        "schema": {
                            "type": "string"
                        },
                        "status": {
                            "description": "The status of this image member",
                            "enum": [
                                "pending",
                                "accepted",
                                "rejected"
                            ],
                            "type": "string"
                        },
                        "updated_at": {
                            "description": ("Date and time of last"
                                            " modification of image member"),
                            "type": "string"
                        }
                    }
                },
                "type": "array"
            },
            "schema": {
                "type": "string"
            }
        }
    }

    def setUp(self):
        super(TestSchemasClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = schemas_client.SchemasClient(fake_auth,
                                                   'image', 'regionOne')

    def _test_show_schema(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_schema,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_SCHEMA,
            bytes_body,
            schema="member")

    def test_show_schema_with_str_body(self):
        self._test_show_schema()

    def test_show_schema_with_bytes_body(self):
        self._test_show_schema(bytes_body=True)



# Copyright 2013 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class FixedIPsNegativeTestJson(base.BaseV2ComputeAdminTest):
    """Negative tests of fixed ips API"""

    @classmethod
    def skip_checks(cls):
        super(FixedIPsNegativeTestJson, cls).skip_checks()
        if CONF.service_available.neutron:
            msg = ("%s skipped as neutron is available" % cls.__name__)
            raise cls.skipException(msg)
        if not utils.get_service_list()['network']:
            raise cls.skipException("network service not enabled.")

    @classmethod
    def setup_clients(cls):
        super(FixedIPsNegativeTestJson, cls).setup_clients()
        cls.client = cls.os_admin.fixed_ips_client
        cls.non_admin_client = cls.fixed_ips_client

    @classmethod
    def resource_setup(cls):
        super(FixedIPsNegativeTestJson, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        server = cls.servers_client.show_server(server['id'])['server']
        cls.ip = None
        for ip_set in server['addresses']:
            for ip in server['addresses'][ip_set]:
                if ip['OS-EXT-IPS:type'] == 'fixed':
                    cls.ip = ip['addr']
                    break
            if cls.ip:
                break
        if cls.ip is None:
            raise cls.skipException("No fixed ip found for server: %s"
                                    % server['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9f17f47d-daad-4adc-986e-12370c93e407')
    def test_list_fixed_ip_details_with_non_admin_user(self):
        """Test listing fixed ip with detail by non-admin user is forbidden"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.show_fixed_ip, self.ip)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ce60042c-fa60-4836-8d43-1c8e3359dc47')
    def test_set_reserve_with_non_admin_user(self):
        """Test reserving fixed ip by non-admin user is forbidden"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.reserve_fixed_ip,
                          self.ip, reserve="None")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f1f7a35b-0390-48c5-9803-5f27461439db')
    def test_set_unreserve_with_non_admin_user(self):
        """Test unreserving fixed ip by non-admin user is forbidden"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_client.reserve_fixed_ip,
                          self.ip, unreserve="None")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f51cf464-7fc5-4352-bc3e-e75cfa2cb717')
    def test_set_reserve_with_invalid_ip(self):
        """Test reserving invalid fixed ip should fail"""
        # NOTE(maurosr): since this exercises the same code snippet, we do it
        # only for reserve action
        # NOTE(eliqiao): in Juno, the exception is NotFound, but in master, we
        # change the error code to BadRequest, both exceptions should be
        # accepted by tempest
        self.assertRaises((lib_exc.NotFound, lib_exc.BadRequest),
                          self.client.reserve_fixed_ip,
                          "my.invalid.ip", reserve="None")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fd26ef50-f135-4232-9d32-281aab3f9176')
    def test_fixed_ip_with_invalid_action(self):
        """Test operating fixed ip with invalid action should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.client.reserve_fixed_ip,
                          self.ip, invalid_action="None")



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

param_network = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'cidr': {'type': ['string', 'null']},
        'label': {'type': 'string'}
    },
    'additionalProperties': False,
    'required': ['id', 'cidr', 'label']
}


list_tenant_networks = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'networks': {
                'type': 'array',
                'items': param_network
            }
        },
        'additionalProperties': False,
        'required': ['networks']
    }
}


get_tenant_network = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'network': param_network
        },
        'additionalProperties': False,
        'required': ['network']
    }
}



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import regions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestRegionsClient(base.BaseServiceTest):
    FAKE_CREATE_REGION = {
        "region": {
            "description": "My subregion",
            "id": "RegionOneSubRegion",
            "parent_region_id": "RegionOne"
            }
        }

    FAKE_REGION_INFO = {
        "region": {
            "description": "My subregion 3",
            "id": "RegionThree",
            "links": {
                "self": "http://example.com/identity/v3/regions/RegionThree"
                },
            "parent_region_id": "RegionOne"
            }
        }

    FAKE_LIST_REGIONS = {
        "links": {
            "next": None,
            "previous": None,
            "self": "http://example.com/identity/v3/regions"
            },
        "regions": [
            {
                "description": "",
                "id": "RegionOne",
                "links": {
                    "self": "http://example.com/identity/v3/regions/RegionOne"
                    },
                "parent_region_id": None
                }
            ]
        }

    def setUp(self):
        super(TestRegionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = regions_client.RegionsClient(fake_auth, 'identity',
                                                   'regionOne')

    def _test_create_region(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_region,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_REGION,
            bytes_body,
            status=201)

    def _test_show_region(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_region,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_REGION_INFO,
            bytes_body,
            region_id="RegionThree")

    def _test_list_regions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_regions,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_REGIONS,
            bytes_body)

    def _test_update_region(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_region,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_REGION_INFO,
            bytes_body,
            region_id="RegionThree")

    def test_create_region_with_str_body(self):
        self._test_create_region()

    def test_create_region_with_bytes_body(self):
        self._test_create_region(bytes_body=True)

    def test_show_region_with_str_body(self):
        self._test_show_region()

    def test_show_region_with_bytes_body(self):
        self._test_show_region(bytes_body=True)

    def test_list_regions_with_str_body(self):
        self._test_list_regions()

    def test_list_regions_with_bytes_body(self):
        self._test_list_regions(bytes_body=True)

    def test_update_region_with_str_body(self):
        self._test_update_region()

    def test_update_region_with_bytes_body(self):
        self._test_update_region(bytes_body=True)

    def test_delete_region(self):
        self.check_service_client_function(
            self.client.delete_region,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            region_id="RegionThree",
            status=204)



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
from unittest import mock

from oslo_utils.fixture import uuidsentinel as uuids

from tempest.common import waiters
from tempest import exceptions
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import servers_client
from tempest.lib.services.volume.v2 import volumes_client
from tempest.tests import base
import tempest.tests.utils as utils


class TestImageWaiters(base.TestCase):
    def setUp(self):
        super(TestImageWaiters, self).setUp()
        self.client = mock.MagicMock()
        self.client.build_timeout = 1
        self.client.build_interval = 1

    def test_wait_for_image_status(self):
        self.client.show_image.return_value = ({'status': 'active'})
        start_time = int(time.time())
        waiters.wait_for_image_status(self.client, 'fake_image_id', 'active')
        end_time = int(time.time())
        # Ensure waiter returns before build_timeout
        self.assertLess((end_time - start_time), 10)

    def test_wait_for_image_status_timeout(self):
        time_mock = self.patch('time.time')
        time_mock.side_effect = utils.generate_timeout_series(1)

        self.client.show_image.return_value = ({'status': 'saving'})
        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_image_status,
                          self.client, 'fake_image_id', 'active')

    def test_wait_for_image_status_error_on_image_create(self):
        self.client.show_image.return_value = ({'status': 'ERROR'})
        self.assertRaises(exceptions.AddImageException,
                          waiters.wait_for_image_status,
                          self.client, 'fake_image_id', 'active')

    def test_wait_for_image_imported_to_stores(self):
        self.client.show_image.return_value = ({'status': 'active',
                                                'stores': 'fake_store'})
        start_time = int(time.time())
        waiters.wait_for_image_imported_to_stores(
            self.client, 'fake_image_id', 'fake_store')
        end_time = int(time.time())
        # Ensure waiter returns before build_timeout
        self.assertLess((end_time - start_time), 10)

    def test_wait_for_image_imported_to_stores_failure(self):
        time_mock = self.patch('time.time')
        client = mock.MagicMock()
        client.build_timeout = 2
        self.patch('time.time', side_effect=[0., 1., 2.])
        time_mock.side_effect = utils.generate_timeout_series(1)

        client.show_image.return_value = ({
            'status': 'saving',
            'stores': 'fake_store',
            'os_glance_failed_import': 'fake_os_glance_failed_import'})
        self.assertRaises(lib_exc.OtherRestClientException,
                          waiters.wait_for_image_imported_to_stores,
                          client, 'fake_image_id', 'fake_store')

    def test_wait_for_image_imported_to_stores_timeout(self):
        time_mock = self.patch('time.time')
        client = mock.MagicMock()
        client.build_timeout = 2
        self.patch('time.time', side_effect=[0., 1., 2.])
        time_mock.side_effect = utils.generate_timeout_series(1)

        client.show_image.return_value = ({
            'status': 'saving',
            'stores': 'fake_store'})
        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_image_imported_to_stores,
                          client, 'fake_image_id', 'fake_store')

    def test_wait_for_image_copied_to_stores(self):
        self.client.show_image.return_value = ({
            'status': 'active',
            'os_glance_importing_to_stores': '',
            'os_glance_failed_import': 'fake_os_glance_failed_import'})
        start_time = int(time.time())
        waiters.wait_for_image_copied_to_stores(
            self.client, 'fake_image_id')
        end_time = int(time.time())
        # Ensure waiter returns before build_timeout
        self.assertLess((end_time - start_time), 10)

    def test_wait_for_image_copied_to_stores_timeout(self):
        time_mock = self.patch('time.time')
        self.patch('time.time', side_effect=[0., 1.])
        time_mock.side_effect = utils.generate_timeout_series(1)

        self.client.show_image.return_value = ({
            'status': 'active',
            'os_glance_importing_to_stores': 'processing',
            'os_glance_failed_import': 'fake_os_glance_failed_import'})
        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_image_copied_to_stores,
                          self.client, 'fake_image_id')


class TestInterfaceWaiters(base.TestCase):

    build_timeout = 1.
    build_interval = 1
    port_down = {'interfaceAttachment': {'port_state': 'DOWN'}}
    port_active = {'interfaceAttachment': {'port_state': 'ACTIVE'}}

    def mock_client(self, **kwargs):
        return mock.MagicMock(
            build_timeout=self.build_timeout,
            build_interval=self.build_interval,
            **kwargs)

    def test_wait_for_interface_status(self):
        show_interface = mock.Mock(
            side_effect=[self.port_down, self.port_active])
        client = self.mock_client(show_interface=show_interface)
        self.patch('time.time', return_value=0.)
        sleep = self.patch('time.sleep')

        result = waiters.wait_for_interface_status(
            client, 'server_id', 'port_id', 'ACTIVE')

        self.assertIs(self.port_active['interfaceAttachment'], result)
        show_interface.assert_has_calls([mock.call('server_id', 'port_id'),
                                         mock.call('server_id', 'port_id')])
        sleep.assert_called_once_with(client.build_interval)

    def test_wait_for_interface_status_timeout(self):
        show_interface = mock.MagicMock(return_value=self.port_down)
        client = self.mock_client(show_interface=show_interface)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        sleep = self.patch('time.sleep')

        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_interface_status,
                          client, 'server_id', 'port_id', 'ACTIVE')

        show_interface.assert_has_calls([mock.call('server_id', 'port_id'),
                                         mock.call('server_id', 'port_id')])
        sleep.assert_called_once_with(client.build_interval)

    one_interface = {'interfaceAttachments': [{'port_id': 'port_one'}]}
    two_interfaces = {'interfaceAttachments': [{'port_id': 'port_one'},
                                               {'port_id': 'port_two'}]}

    def test_wait_for_interface_detach(self):
        list_interfaces = mock.MagicMock(
            side_effect=[self.two_interfaces, self.one_interface])
        client = self.mock_client(list_interfaces=list_interfaces)
        self.patch('time.time', return_value=0.)
        sleep = self.patch('time.sleep')

        result = waiters.wait_for_interface_detach(
            client, 'server_id', 'port_two')

        self.assertIs(self.one_interface['interfaceAttachments'], result)
        list_interfaces.assert_has_calls([mock.call('server_id'),
                                          mock.call('server_id')])
        sleep.assert_called_once_with(client.build_interval)

    def test_wait_for_interface_detach_timeout(self):
        list_interfaces = mock.MagicMock(return_value=self.one_interface)
        client = self.mock_client(list_interfaces=list_interfaces)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        sleep = self.patch('time.sleep')

        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_interface_detach,
                          client, 'server_id', 'port_one')

        list_interfaces.assert_has_calls([mock.call('server_id'),
                                          mock.call('server_id')])
        sleep.assert_called_once_with(client.build_interval)

    def test_wait_for_guest_os_boot(self):
        get_console_output = mock.Mock(
            side_effect=[
                {'output': 'os not ready yet\n'},
                {'output': 'login:\n'}
            ])
        client = self.mock_client(get_console_output=get_console_output)
        self.patch('time.time', return_value=0.)
        sleep = self.patch('time.sleep')

        with mock.patch.object(waiters.LOG, "info") as log_info:
            waiters.wait_for_guest_os_boot(client, 'server_id')

        get_console_output.assert_has_calls([
            mock.call('server_id'), mock.call('server_id')])
        sleep.assert_called_once_with(client.build_interval)
        log_info.assert_not_called()

    def test_wait_for_guest_os_boot_timeout(self):
        get_console_output = mock.Mock(
            return_value={'output': 'os not ready yet\n'})
        client = self.mock_client(get_console_output=get_console_output)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        self.patch('time.sleep')

        with mock.patch.object(waiters.LOG, "info") as log_info:
            waiters.wait_for_guest_os_boot(client, 'server_id')

        log_info.assert_called_once()


class TestVolumeWaiters(base.TestCase):
    vol_migrating_src_host = {
        'volume': {'migration_status': 'migrating',
                   'os-vol-host-attr:host': 'src_host@backend#type'}}
    vol_migrating_dst_host = {
        'volume': {'migration_status': 'migrating',
                   'os-vol-host-attr:host': 'dst_host@backend#type'}}
    vol_migration_success = {
        'volume': {'migration_status': 'success',
                   'os-vol-host-attr:host': 'dst_host@backend#type'}}
    vol_migration_error = {
        'volume': {'migration_status': 'error',
                   'os-vol-host-attr:host': 'src_host@backend#type'}}

    def test_wait_for_volume_migration_timeout(self):
        show_volume = mock.MagicMock(return_value=self.vol_migrating_src_host)
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           resource_type="volume",
                           build_interval=1,
                           build_timeout=1,
                           show_volume=show_volume)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        self.patch('time.sleep')
        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_volume_migration,
                          client, mock.sentinel.volume_id, 'dst_host')

    def test_wait_for_volume_migration_error(self):
        show_volume = mock.MagicMock(side_effect=[
            self.vol_migrating_src_host,
            self.vol_migrating_src_host,
            self.vol_migration_error])
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           resource_type="volume",
                           build_interval=1,
                           build_timeout=1,
                           show_volume=show_volume)
        self.patch('time.time', return_value=0.)
        self.patch('time.sleep')
        self.assertRaises(lib_exc.TempestException,
                          waiters.wait_for_volume_migration,
                          client, mock.sentinel.volume_id, 'dst_host')

    def test_wait_for_volume_migration_success_and_dst(self):
        show_volume = mock.MagicMock(side_effect=[
            self.vol_migrating_src_host,
            self.vol_migrating_dst_host,
            self.vol_migration_success])
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           resource_type="volume",
                           build_interval=1,
                           build_timeout=1,
                           show_volume=show_volume)
        self.patch('time.time', return_value=0.)
        self.patch('time.sleep')
        waiters.wait_for_volume_migration(
            client, mock.sentinel.volume_id, 'dst_host')

        # Assert that we wait until migration_status is success and dst_host is
        # part of the returned os-vol-host-attr:host.
        show_volume.assert_has_calls([mock.call(mock.sentinel.volume_id),
                                      mock.call(mock.sentinel.volume_id),
                                      mock.call(mock.sentinel.volume_id)])

    @mock.patch.object(time, 'sleep')
    def test_wait_for_volume_status_error_restoring(self, mock_sleep):
        # Tests that the wait method raises VolumeRestoreErrorException if
        # the volume status is 'error_restoring'.
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           resource_type="volume",
                           build_interval=1)
        volume1 = {'volume': {'status': 'restoring-backup'}}
        volume2 = {'volume': {'status': 'error_restoring'}}
        mock_show = mock.Mock(side_effect=(volume1, volume2))
        client.show_volume = mock_show
        volume_id = '7532b91e-aa0a-4e06-b3e5-20c0c5ee1caa'
        self.assertRaises(exceptions.VolumeRestoreErrorException,
                          waiters.wait_for_volume_resource_status,
                          client, volume_id, 'available')
        mock_show.assert_has_calls([mock.call(volume_id),
                                    mock.call(volume_id)])
        mock_sleep.assert_called_once_with(1)

    @mock.patch.object(time, 'sleep')
    def test_wait_for_volume_status_error_extending(self, mock_sleep):
        # Tests that the wait method raises VolumeExtendErrorException if
        # the volume status is 'error_extending'.
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           resource_type="volume",
                           build_interval=1)
        volume1 = {'volume': {'status': 'extending'}}
        volume2 = {'volume': {'status': 'error_extending'}}
        mock_show = mock.Mock(side_effect=(volume1, volume2))
        client.show_volume = mock_show
        volume_id = '7532b91e-aa0a-4e06-b3e5-20c0c5ee1caa'
        self.assertRaises(exceptions.VolumeExtendErrorException,
                          waiters.wait_for_volume_resource_status,
                          client, volume_id, 'available')
        mock_show.assert_has_calls([mock.call(volume_id),
                                    mock.call(volume_id)])
        mock_sleep.assert_called_once_with(1)

    def test_wait_for_volume_attachment_create(self):
        vol_detached = {'volume': {'attachments': []}}
        vol_attached = {'volume': {'attachments': [
                       {'id': uuids.volume_id,
                        'attachment_id': uuids.attachment_id,
                        'server_id': uuids.server_id,
                        'volume_id': uuids.volume_id}]}}
        show_volume = mock.MagicMock(side_effect=[
            vol_detached, vol_detached, vol_attached])
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           build_interval=1,
                           build_timeout=5,
                           show_volume=show_volume)
        self.patch('time.time')
        self.patch('time.sleep')
        att = waiters.wait_for_volume_attachment_create(
            client, uuids.volume_id, uuids.server_id)
        assert att == vol_attached['volume']['attachments'][0]
        # Assert that show volume is called until the attachment is removed.
        show_volume.assert_has_calls([mock.call(uuids.volume_id),
                                      mock.call(uuids.volume_id),
                                      mock.call(uuids.volume_id)])

    def test_wait_for_volume_attachment(self):
        vol_detached = {'volume': {'attachments': []}}
        vol_attached = {'volume': {'attachments': [
                       {'attachment_id': uuids.attachment_id}]}}
        show_volume = mock.MagicMock(side_effect=[
            vol_attached, vol_attached, vol_detached])
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           build_interval=1,
                           build_timeout=5,
                           show_volume=show_volume)
        self.patch('time.time')
        self.patch('time.sleep')
        waiters.wait_for_volume_attachment_remove(client, uuids.volume_id,
                                                  uuids.attachment_id)
        # Assert that show volume is called until the attachment is removed.
        show_volume.assert_has_calls([mock.call(uuids.volume_id),
                                      mock.call(uuids.volume_id),
                                      mock.call(uuids.volume_id)])

    def test_wait_for_volume_attachment_timeout(self):
        show_volume = mock.MagicMock(return_value={
            'volume': {'attachments': [
                {'attachment_id': uuids.attachment_id}]}})
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           build_interval=1,
                           build_timeout=1,
                           show_volume=show_volume)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        self.patch('time.sleep')
        # Assert that a timeout is raised if the attachment remains.
        self.assertRaises(lib_exc.TimeoutException,
                          waiters.wait_for_volume_attachment_remove,
                          client, uuids.volume_id, uuids.attachment_id)

    def test_wait_for_volume_attachment_not_present(self):
        show_volume = mock.MagicMock(return_value={
            'volume': {'attachments': []}})
        client = mock.Mock(spec=volumes_client.VolumesClient,
                           build_interval=1,
                           build_timeout=1,
                           show_volume=show_volume)
        self.patch('time.time', side_effect=[0., client.build_timeout + 1.])
        self.patch('time.sleep')
        waiters.wait_for_volume_attachment_remove(client, uuids.volume_id,
                                                  uuids.attachment_id)
        # Assert that show volume is only called once before we return
        show_volume.assert_called_once_with(uuids.volume_id)

    def test_wait_for_volume_attachment_remove_from_server(self):
        volume_attached = {
            "volumeAttachments": [{"volumeId": uuids.volume_id}]}
        volume_not_attached = {"volumeAttachments": []}
        mock_list_volume_attachments = mock.Mock(
            side_effect=[volume_attached, volume_not_attached])
        mock_client = mock.Mock(
            spec=servers_client.ServersClient,
            build_interval=1,
            build_timeout=1,
            list_volume_attachments=mock_list_volume_attachments)
        self.patch(
            'time.time',
            side_effect=[0., 0.5, mock_client.build_timeout + 1.])
        self.patch('time.sleep')

        waiters.wait_for_volume_attachment_remove_from_server(
            mock_client, uuids.server_id, uuids.volume_id)

        # Assert that list_volume_attachments is called until the attachment is
        # removed.
        mock_list_volume_attachments.assert_has_calls([
            mock.call(uuids.server_id),
            mock.call(uuids.server_id)])

    def test_wait_for_volume_attachment_remove_from_server_timeout(self):
        volume_attached = {
            "volumeAttachments": [{"volumeId": uuids.volume_id}]}
        mock_list_volume_attachments = mock.Mock(
            side_effect=[volume_attached, volume_attached])
        mock_client = mock.Mock(
            spec=servers_client.ServersClient,
            build_interval=1,
            build_timeout=1,
            list_volume_attachments=mock_list_volume_attachments)
        self.patch(
            'time.time',
            side_effect=[0., 0.5, mock_client.build_timeout + 1.])
        self.patch('time.sleep')

        self.assertRaises(
            lib_exc.TimeoutException,
            waiters.wait_for_volume_attachment_remove_from_server,
            mock_client, uuids.server_id, uuids.volume_id)

        # Assert that list_volume_attachments is called until the attachment is
        # removed.
        mock_list_volume_attachments.assert_has_calls([
            mock.call(uuids.server_id),
            mock.call(uuids.server_id)])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import limits as schemav21
from tempest.lib.api_schema.response.compute.v2_36 import limits as schemav236
from tempest.lib.api_schema.response.compute.v2_39 import limits as schemav239
from tempest.lib.api_schema.response.compute.v2_57 import limits as schemav257
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class LimitsClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.35', 'schema': schemav21},
        {'min': '2.36', 'max': '2.38', 'schema': schemav236},
        {'min': '2.39', 'max': '2.56', 'schema': schemav239},
        {'min': '2.57', 'max': None, 'schema': schemav257}]

    def show_limits(self):
        resp, body = self.get("limits")
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_limit, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions as lib_exc
from tempest.lib.services.network import base


class SecurityGroupsClient(base.BaseNetworkClient):

    def create_security_group(self, **kwargs):
        """Creates an OpenStack Networking security group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-security-group
        """
        uri = '/security-groups'
        post_data = {'security_group': kwargs}
        return self.create_resource(uri, post_data)

    def update_security_group(self, security_group_id, **kwargs):
        """Updates a security group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-security-group
        """
        uri = '/security-groups/%s' % security_group_id
        post_data = {'security_group': kwargs}
        return self.update_resource(uri, post_data)

    def show_security_group(self, security_group_id, **fields):
        """Shows details for a security group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-security-group
        """
        uri = '/security-groups/%s' % security_group_id
        return self.show_resource(uri, **fields)

    def delete_security_group(self, security_group_id):
        """Deletes an OpenStack Networking security group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-security-group
        """
        uri = '/security-groups/%s' % security_group_id
        return self.delete_resource(uri)

    def list_security_groups(self, **filters):
        """Lists OpenStack Networking security groups.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-security-groups
        """
        uri = '/security-groups'
        return self.list_resources(uri, **filters)

    def is_resource_deleted(self, id):
        try:
            self.show_security_group(id)
        except lib_exc.NotFound:
            return True
        return False



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import utils
from tempest import config
from tempest.lib import decorators


CONF = config.CONF


class VolumesCloneTest(base.BaseVolumeTest):
    """Test volume clone"""

    @classmethod
    def skip_checks(cls):
        super(VolumesCloneTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.clone:
            raise cls.skipException("Cinder volume clones are disabled")

    def _verify_volume_clone(self, source_volume, cloned_volume,
                             bootable='false', extra_size=0):

        cloned_vol_details = self.volumes_client.show_volume(
            cloned_volume['id'])['volume']

        self.assertEqual(source_volume['id'],
                         cloned_vol_details['source_volid'])
        self.assertEqual(source_volume['size'] + extra_size,
                         cloned_vol_details['size'])
        self.assertEqual(bootable, cloned_vol_details['bootable'])

    @decorators.idempotent_id('9adae371-a257-43a5-9555-dc7c88e66e0e')
    def test_create_from_volume(self):
        """Test cloning a volume with increasing size"""
        # Creates a volume from another volume passing a size different from
        # the source volume.
        src_size = CONF.volume.volume_size

        src_vol = self.create_volume(size=src_size)
        # Destination volume bigger than source
        dst_vol = self.create_volume(source_volid=src_vol['id'],
                                     size=src_size + 1)

        self._verify_volume_clone(src_vol, dst_vol, extra_size=1)

    @decorators.idempotent_id('cbbcd7c6-5a6c-481a-97ac-ca55ab715d16')
    @utils.services('image')
    def test_create_from_bootable_volume(self):
        """Test cloning a bootable volume"""
        # Create volume from image
        img_uuid = CONF.compute.image_ref
        src_vol = self.create_volume(imageRef=img_uuid)

        # Create a volume from the bootable volume
        cloned_vol = self.create_volume(source_volid=src_vol['id'])

        self._verify_volume_clone(src_vol, cloned_vol, bootable='true')



# Copyright 2017 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_config import cfg
import testtools

from tempest.common import credentials_factory as cf
from tempest import config
from tempest.lib.common import dynamic_creds
from tempest.lib.common import preprov_creds
from tempest.lib import exceptions
from tempest.tests import base
from tempest.tests import fake_config
from tempest.tests.lib import fake_credentials


class TestCredentialsFactory(base.TestCase):

    def setUp(self):
        super(TestCredentialsFactory, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)

    def test_get_dynamic_provider_params_creds_v2(self):
        expected_uri = 'EXPECTED_V2_URI'
        cfg.CONF.set_default('uri', expected_uri, group='identity')
        admin_creds = fake_credentials.FakeCredentials()
        params = cf.get_dynamic_provider_params('v2', admin_creds=admin_creds)
        expected_params = dict(identity_uri=expected_uri,
                               admin_creds=admin_creds)
        for key in expected_params:
            self.assertIn(key, params)
            self.assertEqual(expected_params[key], params[key])

    def test_get_dynamic_provider_params_creds_v3(self):
        expected_uri = 'EXPECTED_V3_URI'
        cfg.CONF.set_default('uri_v3', expected_uri, group='identity')
        admin_creds = fake_credentials.FakeCredentials()
        params = cf.get_dynamic_provider_params('v3', admin_creds=admin_creds)
        expected_params = dict(identity_uri=expected_uri,
                               admin_creds=admin_creds)
        for key in expected_params:
            self.assertIn(key, params)
            self.assertEqual(expected_params[key], params[key])

    def test_get_dynamic_provider_params_creds_vx(self):
        admin_creds = fake_credentials.FakeCredentials()
        invalid_version = 'invalid_version_x'
        with testtools.ExpectedException(
                exc_type=exceptions.InvalidIdentityVersion,
                value_re='Invalid version ' + invalid_version):
            cf.get_dynamic_provider_params(invalid_version,
                                           admin_creds=admin_creds)

    def test_get_dynamic_provider_params_no_creds(self):
        expected_identity_version = 'v3'
        with mock.patch.object(
                cf, 'get_configured_admin_credentials') as admin_creds_mock:
            cf.get_dynamic_provider_params(expected_identity_version)
            admin_creds_mock.assert_called_once_with(
                fill_in=True, identity_version=expected_identity_version)

    def test_get_preprov_provider_params_creds_v2(self):
        expected_uri = 'EXPECTED_V2_URI'
        cfg.CONF.set_default('uri', expected_uri, group='identity')
        params = cf.get_preprov_provider_params('v2')
        self.assertIn('identity_uri', params)
        self.assertEqual(expected_uri, params['identity_uri'])

    def test_get_preprov_provider_params_creds_v3(self):
        expected_uri = 'EXPECTED_V3_URI'
        cfg.CONF.set_default('uri_v3', expected_uri, group='identity')
        params = cf.get_preprov_provider_params('v3')
        self.assertIn('identity_uri', params)
        self.assertEqual(expected_uri, params['identity_uri'])

    def test_get_preprov_provider_params_creds_vx(self):
        invalid_version = 'invalid_version_x'
        with testtools.ExpectedException(
                exc_type=exceptions.InvalidIdentityVersion,
                value_re='Invalid version ' + invalid_version):
            cf.get_dynamic_provider_params(invalid_version)

    @mock.patch.object(dynamic_creds, 'DynamicCredentialProvider')
    @mock.patch.object(cf, 'get_dynamic_provider_params')
    def test_get_credentials_provider_dynamic(
            self, mock_dynamic_provider_params,
            mock_dynamic_credentials_provider_class):
        cfg.CONF.set_default('use_dynamic_credentials', True, group='auth')
        expected_params = {'foo': 'bar'}
        mock_dynamic_provider_params.return_value = expected_params
        expected_name = 'my_name'
        expected_network_resources = {'network': 'resources'}
        expected_identity_version = 'identity_version'
        cf.get_credentials_provider(
            expected_name,
            network_resources=expected_network_resources,
            force_tenant_isolation=False,
            identity_version=expected_identity_version)
        mock_dynamic_provider_params.assert_called_once_with(
            expected_identity_version)
        mock_dynamic_credentials_provider_class.assert_called_once_with(
            name=expected_name, network_resources=expected_network_resources,
            **expected_params)

    @mock.patch.object(preprov_creds, 'PreProvisionedCredentialProvider')
    @mock.patch.object(cf, 'get_preprov_provider_params')
    def test_get_credentials_provider_preprov(
            self, mock_preprov_provider_params,
            mock_preprov_credentials_provider_class):
        cfg.CONF.set_default('use_dynamic_credentials', False, group='auth')
        cfg.CONF.set_default('test_accounts_file', '/some/file', group='auth')
        expected_params = {'foo': 'bar'}
        mock_preprov_provider_params.return_value = expected_params
        expected_name = 'my_name'
        expected_identity_version = 'identity_version'
        cf.get_credentials_provider(
            expected_name,
            force_tenant_isolation=False,
            identity_version=expected_identity_version)
        mock_preprov_provider_params.assert_called_once_with(
            expected_identity_version)
        mock_preprov_credentials_provider_class.assert_called_once_with(
            name=expected_name, **expected_params)

    def test_get_credentials_provider_preprov_no_file(self):
        cfg.CONF.set_default('use_dynamic_credentials', False, group='auth')
        cfg.CONF.set_default('test_accounts_file', None, group='auth')
        with testtools.ExpectedException(
                exc_type=exceptions.InvalidConfiguration):
            cf.get_credentials_provider(
                'some_name',
                force_tenant_isolation=False,
                identity_version='some_version')

    @mock.patch.object(dynamic_creds, 'DynamicCredentialProvider')
    @mock.patch.object(cf, 'get_dynamic_provider_params')
    def test_get_credentials_provider_force_dynamic(
            self, mock_dynamic_provider_params,
            mock_dynamic_credentials_provider_class):
        cfg.CONF.set_default('use_dynamic_credentials', False, group='auth')
        expected_params = {'foo': 'bar'}
        mock_dynamic_provider_params.return_value = expected_params
        expected_name = 'my_name'
        expected_network_resources = {'network': 'resources'}
        expected_identity_version = 'identity_version'
        cf.get_credentials_provider(
            expected_name,
            network_resources=expected_network_resources,
            force_tenant_isolation=True,
            identity_version=expected_identity_version)
        mock_dynamic_provider_params.assert_called_once_with(
            expected_identity_version)
        mock_dynamic_credentials_provider_class.assert_called_once_with(
            name=expected_name, network_resources=expected_network_resources,
            **expected_params)

    @mock.patch.object(cf, 'get_credentials')
    def test_get_configured_admin_credentials(self, mock_get_credentials):
        cfg.CONF.set_default('auth_version', 'v3', 'identity')
        all_params = [
            ('admin_username', 'username', 'my_name'),
            ('admin_user_domain_name', 'user_domain_name', 'my_dname'),
            ('admin_password', 'password', 'secret'),
            ('admin_project_name', 'project_name', 'my_pname'),
            ('admin_project_domain_name', 'project_domain_name', 'my_dname'),
            ('admin_domain_name', 'domain_name', 'my_dname'),
            ('admin_system', 'system', None),
        ]
        expected_result = 'my_admin_credentials'
        mock_get_credentials.return_value = expected_result
        for config_item, _, value in all_params:
            cfg.CONF.set_default(config_item, value, 'auth')
        # Build the expected params
        expected_params = dict(
            [(field, value) for _, field, value in all_params])
        expected_params.update(config.service_client_config())
        admin_creds = cf.get_configured_admin_credentials()
        mock_get_credentials.assert_called_once_with(
            fill_in=True, identity_version='v3', **expected_params)
        self.assertEqual(expected_result, admin_creds)

    @mock.patch.object(cf, 'get_credentials')
    def test_get_configured_admin_credentials_not_fill_valid(
            self, mock_get_credentials):
        cfg.CONF.set_default('auth_version', 'v2', 'identity')
        all_params = [
            ('admin_username', 'username', 'my_name'),
            ('admin_user_domain_name', 'user_domain_name', 'my_dname'),
            ('admin_password', 'password', 'secret'),
            ('admin_project_domain_name', 'project_domain_name', 'my_dname'),
            ('admin_project_name', 'project_name', 'my_pname'),
            ('admin_domain_name', 'domain_name', 'my_dname'),
            ('admin_system', 'system', None),
        ]
        expected_result = mock.Mock()
        expected_result.is_valid.return_value = True
        mock_get_credentials.return_value = expected_result
        for config_item, _, value in all_params:
            cfg.CONF.set_default(config_item, value, 'auth')
        # Build the expected params
        expected_params = dict(
            [(field, value) for _, field, value in all_params])
        expected_params.update(config.service_client_config())
        admin_creds = cf.get_configured_admin_credentials(
            fill_in=False, identity_version='v3')
        mock_get_credentials.assert_called_once_with(
            fill_in=False, identity_version='v3', **expected_params)
        self.assertEqual(expected_result, admin_creds)
        expected_result.is_valid.assert_called_once()

    @mock.patch.object(cf, 'get_credentials')
    def test_get_configured_admin_credentials_not_fill_not_valid(
            self, mock_get_credentials):
        cfg.CONF.set_default('auth_version', 'v2', 'identity')
        expected_result = mock.Mock()
        expected_result.is_valid.return_value = False
        mock_get_credentials.return_value = expected_result
        with testtools.ExpectedException(exceptions.InvalidConfiguration,
                                         value_re='.*\n.*identity version v2'):
            cf.get_configured_admin_credentials(fill_in=False)

    @mock.patch('tempest.lib.auth.get_credentials')
    def test_get_credentials_v2(self, mock_auth_get_credentials):
        expected_uri = 'V2_URI'
        expected_result = 'my_creds'
        mock_auth_get_credentials.return_value = expected_result
        cfg.CONF.set_default('uri', expected_uri, 'identity')
        params = {'foo': 'bar'}
        expected_params = params.copy()
        expected_params.update(config.service_client_config())
        result = cf.get_credentials(identity_version='v2', **params)
        self.assertEqual(expected_result, result)
        mock_auth_get_credentials.assert_called_once_with(
            expected_uri, fill_in=True, identity_version='v2',
            **expected_params)

    @mock.patch('tempest.lib.auth.get_credentials')
    def test_get_credentials_v3_no_domain(self, mock_auth_get_credentials):
        expected_uri = 'V3_URI'
        expected_result = 'my_creds'
        expected_domain = 'my_domain'
        mock_auth_get_credentials.return_value = expected_result
        cfg.CONF.set_default('uri_v3', expected_uri, 'identity')
        cfg.CONF.set_default('default_credentials_domain_name',
                             expected_domain, 'auth')
        params = {'foo': 'bar'}
        expected_params = params.copy()
        expected_params['domain_name'] = expected_domain
        expected_params.update(config.service_client_config())
        result = cf.get_credentials(fill_in=False, identity_version='v3',
                                    **params)
        self.assertEqual(expected_result, result)
        mock_auth_get_credentials.assert_called_once_with(
            expected_uri, fill_in=False, identity_version='v3',
            **expected_params)

    @mock.patch('tempest.lib.auth.get_credentials')
    def test_get_credentials_v3_domain(self, mock_auth_get_credentials):
        expected_uri = 'V3_URI'
        expected_result = 'my_creds'
        expected_domain = 'my_domain'
        mock_auth_get_credentials.return_value = expected_result
        cfg.CONF.set_default('uri_v3', expected_uri, 'identity')
        cfg.CONF.set_default('default_credentials_domain_name',
                             expected_domain, 'auth')
        params = {'foo': 'bar', 'user_domain_name': expected_domain}
        expected_params = params.copy()
        expected_params.update(config.service_client_config())
        result = cf.get_credentials(fill_in=False, identity_version='v3',
                                    **params)
        self.assertEqual(expected_result, result)
        mock_auth_get_credentials.assert_called_once_with(
            expected_uri, fill_in=False, identity_version='v3',
            **expected_params)

    @mock.patch('tempest.lib.auth.get_credentials')
    def test_get_credentials_v3_system(self, mock_auth_get_credentials):
        expected_uri = 'V3_URI'
        expected_result = 'my_creds'
        mock_auth_get_credentials.return_value = expected_result
        cfg.CONF.set_default('uri_v3', expected_uri, 'identity')
        cfg.CONF.set_default('admin_system', 'all', 'auth')
        params = {'system': 'all'}
        expected_params = params.copy()
        expected_params.update(config.service_client_config())
        result = cf.get_credentials(fill_in=False, identity_version='v3',
                                    **params)
        self.assertEqual(expected_result, result)
        mock_auth_get_credentials.assert_called_once_with(
            expected_uri, fill_in=False, identity_version='v3',
            **expected_params)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class QuotasAdminNegativeTestBase(base.BaseV2ComputeAdminTest):
    force_tenant_isolation = True

    @classmethod
    def setup_clients(cls):
        super(QuotasAdminNegativeTestBase, cls).setup_clients()
        cls.client = cls.os_primary.quotas_client
        cls.adm_client = cls.os_admin.quotas_client
        cls.sg_client = cls.security_groups_client
        cls.sgr_client = cls.security_group_rules_client

    @classmethod
    def resource_setup(cls):
        super(QuotasAdminNegativeTestBase, cls).resource_setup()
        # NOTE(afazekas): these test cases should always create and use a new
        # tenant most of them should be skipped if we can't do that
        cls.demo_tenant_id = cls.client.tenant_id

    def _update_quota(self, quota_item, quota_value):
        quota_set = (self.adm_client.show_quota_set(self.demo_tenant_id)
                     ['quota_set'])
        default_quota_value = quota_set[quota_item]

        self.adm_client.update_quota_set(self.demo_tenant_id,
                                         force=True,
                                         **{quota_item: quota_value})
        self.addCleanup(self.adm_client.update_quota_set, self.demo_tenant_id,
                        **{quota_item: default_quota_value})


class QuotasAdminNegativeTest(QuotasAdminNegativeTestBase):
    """Negative tests of nova quotas"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('733abfe8-166e-47bb-8363-23dbd7ff3476')
    def test_update_quota_normal_user(self):
        """Test updating nova quota by normal user should fail"""
        self.assertRaises(lib_exc.Forbidden,
                          self.client.update_quota_set,
                          self.demo_tenant_id,
                          ram=0)

    # TODO(afazekas): Add dedicated tenant to the skipped quota tests.
    # It can be moved into the setUpClass as well.
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('91058876-9947-4807-9f22-f6eb17140d9b')
    def test_create_server_when_cpu_quota_is_full(self):
        """Disallow server creation when tenant's vcpu quota is full"""
        self._update_quota('cores', 0)
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.create_test_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6fdd7012-584d-4327-a61c-49122e0d5864')
    def test_create_server_when_memory_quota_is_full(self):
        """Disallow server creation when tenant's memory quota is full"""
        self._update_quota('ram', 0)
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.create_test_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7c6be468-0274-449a-81c3-ac1c32ee0161')
    def test_create_server_when_instances_quota_is_full(self):
        """Once instances quota limit is reached, disallow server creation"""
        self._update_quota('instances', 0)
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.create_test_server)


class QuotasSecurityGroupAdminNegativeTest(QuotasAdminNegativeTestBase):
    """Negative tests of nova security group quota"""

    max_microversion = '2.35'

    @decorators.skip_because(bug="1186354",
                             condition=CONF.service_available.neutron)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7c6c8f3b-2bf6-4918-b240-57b136a66aa0')
    @utils.services('network')
    def test_security_groups_exceed_limit(self):
        """Negative test: Creation Security Groups over limit should FAIL"""
        # Set the quota to number of used security groups
        sg_quota = self.limits_client.show_limits()['limits']['absolute'][
            'totalSecurityGroupsUsed']
        self._update_quota('security_groups', sg_quota)

        # Check we cannot create anymore
        # A 403 Forbidden or 413 Overlimit (old behaviour) exception
        # will be raised when out of quota
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.sg_client.create_security_group,
                          name="sg-overlimit", description="sg-desc")

    @decorators.skip_because(bug="1186354",
                             condition=CONF.service_available.neutron)
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6e9f436d-f1ed-4f8e-a493-7275dfaa4b4d')
    @utils.services('network')
    def test_security_groups_rules_exceed_limit(self):
        """Negative test: Creation of Security Group Rules should FAIL"""
        # when we reach limit maxSecurityGroupRules
        self._update_quota('security_group_rules', 0)

        s_name = data_utils.rand_name('securitygroup')
        s_description = data_utils.rand_name('description')
        securitygroup = self.sg_client.create_security_group(
            name=s_name, description=s_description)['security_group']
        self.addCleanup(self.sg_client.delete_security_group,
                        securitygroup['id'])

        secgroup_id = securitygroup['id']
        ip_protocol = 'tcp'

        # Check we cannot create SG rule anymore
        # A 403 Forbidden or 413 Overlimit (old behaviour) exception
        # will be raised when out of quota
        self.assertRaises((lib_exc.OverLimit, lib_exc.Forbidden),
                          self.sgr_client.create_security_group_rule,
                          parent_group_id=secgroup_id, ip_protocol=ip_protocol,
                          from_port=1025, to_port=1025)



# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ServerRescueTestBase(base.BaseV2ComputeTest):
    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(ServerRescueTestBase, cls).skip_checks()
        if not CONF.compute_feature_enabled.rescue:
            msg = "Server rescue not available."
            raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True)
        super(ServerRescueTestBase, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerRescueTestBase, cls).resource_setup()

        password = data_utils.rand_password()
        server = cls.create_test_server(adminPass=password,
                                        wait_until='ACTIVE')
        cls.servers_client.rescue_server(server['id'], adminPass=password)
        waiters.wait_for_server_status(cls.servers_client, server['id'],
                                       'RESCUE')
        cls.rescued_server_id = server['id']


class ServerRescueTestJSON(ServerRescueTestBase):
    """Test server rescue"""

    @decorators.idempotent_id('fd032140-714c-42e4-a8fd-adcd8df06be6')
    def test_rescue_unrescue_instance(self):
        """Test rescue/unrescue server"""
        password = data_utils.rand_password()
        server = self.create_test_server(adminPass=password,
                                         wait_until='ACTIVE')
        self.servers_client.rescue_server(server['id'], adminPass=password)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'RESCUE')
        self.servers_client.unrescue_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')


class ServerRescueTestJSONUnderV235(ServerRescueTestBase):
    """Test server rescue with compute microversion less than 2.36"""

    max_microversion = '2.35'

    # TODO(zhufl): After 2.35 we should switch to neutron client to create
    # floating ip, but that will need admin credential, so the testcases will
    # have to be added in somewhere in admin directory.

    @decorators.idempotent_id('4842e0cf-e87d-4d9d-b61f-f4791da3cacc')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @testtools.skipUnless(CONF.network_feature_enabled.floating_ips,
                          "Floating ips are not available")
    def test_rescued_vm_associate_dissociate_floating_ip(self):
        """Test associate/dissociate floating ip for rescued server"""
        floating_ip_body = self.floating_ips_client.create_floating_ip(
            pool=CONF.network.floating_network_name)['floating_ip']
        self.addCleanup(self.floating_ips_client.delete_floating_ip,
                        floating_ip_body['id'])

        self.floating_ips_client.associate_floating_ip_to_server(
            str(floating_ip_body['ip']).strip(), self.rescued_server_id)

        # Disassociation of floating IP that was associated in this method
        self.floating_ips_client.disassociate_floating_ip_from_server(
            str(floating_ip_body['ip']).strip(), self.rescued_server_id)

    @decorators.idempotent_id('affca41f-7195-492d-8065-e09eee245404')
    def test_rescued_vm_add_remove_security_group(self):
        """Test add/remove security group to for rescued server"""
        # Add Security group
        sg = self.create_security_group()
        self.servers_client.add_security_group(self.rescued_server_id,
                                               name=sg['name'])

        # Delete Security group
        self.servers_client.remove_security_group(self.rescued_server_id,
                                                  name=sg['name'])


class BaseServerStableDeviceRescueTest(base.BaseV2ComputeTest):
    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(BaseServerStableDeviceRescueTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.rescue:
            msg = "Server rescue not available."
            raise cls.skipException(msg)
        if not CONF.compute_feature_enabled.stable_rescue:
            msg = "Stable rescue not available."
            raise cls.skipException(msg)

    def _create_server_and_rescue_image(self, hw_rescue_device=None,
                                        hw_rescue_bus=None,
                                        block_device_mapping_v2=None):

        server_id = self.create_test_server(
            wait_until='ACTIVE')['id']
        image_id = self.create_image_from_server(
            server_id, wait_until='ACTIVE')['id']

        if block_device_mapping_v2:
            server_id = self.create_test_server(
                wait_until='ACTIVE',
                block_device_mapping_v2=block_device_mapping_v2)['id']

        if hw_rescue_bus:
            self.images_client.update_image(
                image_id, [dict(add='/hw_rescue_bus',
                                value=hw_rescue_bus)])
        if hw_rescue_device:
            self.images_client.update_image(
                image_id, [dict(add='/hw_rescue_device',
                                value=hw_rescue_device)])
        return server_id, image_id

    def _test_stable_device_rescue(self, server_id, rescue_image_id):
        self.servers_client.rescue_server(
            server_id, rescue_image_ref=rescue_image_id)
        waiters.wait_for_server_status(
            self.servers_client, server_id, 'RESCUE')
        self.servers_client.unrescue_server(server_id)
        waiters.wait_for_server_status(
            self.servers_client, server_id, 'ACTIVE')


class ServerStableDeviceRescueTestIDE(BaseServerStableDeviceRescueTest):
    """Test rescuing server using an IDE device for the rescue disk"""

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.compute_feature_enabled.ide_bus:
            raise cls.skipException("IDE bus not available.")

    @decorators.idempotent_id('947004c3-e8ef-47d9-9f00-97b74f9eaf96')
    def test_stable_device_rescue_cdrom_ide(self):
        """Test rescuing server with cdrom and ide as the rescue disk"""
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='cdrom', hw_rescue_bus='ide')
        self._test_stable_device_rescue(server_id, rescue_image_id)


class ServerStableDeviceRescueTest(BaseServerStableDeviceRescueTest):
    """Test rescuing server specifying type of device for the rescue disk"""

    @decorators.idempotent_id('16865750-1417-4854-bcf7-496e6753c01e')
    def test_stable_device_rescue_disk_virtio(self):
        """Test rescuing server with disk and virtio as the rescue disk"""
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='virtio')
        self._test_stable_device_rescue(server_id, rescue_image_id)

    @decorators.idempotent_id('12340157-6306-4745-bdda-cfa019908b48')
    def test_stable_device_rescue_disk_scsi(self):
        """Test rescuing server with disk and scsi as the rescue disk"""
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='scsi')
        self._test_stable_device_rescue(server_id, rescue_image_id)

    @decorators.idempotent_id('647d04cf-ad35-4956-89ab-b05c5c16f30c')
    def test_stable_device_rescue_disk_usb(self):
        """Test rescuing server with disk and usb as the rescue disk"""
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='usb')
        self._test_stable_device_rescue(server_id, rescue_image_id)

    @decorators.idempotent_id('a3772b42-00bf-4310-a90b-1cc6fd3e7eab')
    @utils.services('volume')
    def test_stable_device_rescue_disk_virtio_with_volume_attached(self):
        """Test rescuing server with volume attached

        Attach a volume to the server and then rescue the server with disk
        and virtio as the rescue disk.
        """
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='virtio')
        server = self.servers_client.show_server(server_id)['server']
        volume = self.create_volume()
        self.attach_volume(server, volume)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'in-use')
        self._test_stable_device_rescue(server_id, rescue_image_id)


class ServerBootFromVolumeStableRescueTest(BaseServerStableDeviceRescueTest):
    """Test rescuing server specifying type of device for the rescue disk

    Test rescuing server specifying type of device for the rescue disk with
    compute microversion greater than 2.86.
    """

    min_microversion = '2.87'

    @classmethod
    def skip_checks(cls):
        super(ServerBootFromVolumeStableRescueTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('48f123cb-922a-4065-8db6-b9a9074a556b')
    def test_stable_device_rescue_bfv_blank_volume(self):
        """Test rescuing server with blank volume as block_device_mapping_v2

        Create a server with block_device_mapping_v2 with blank volume,
        then rescue the server with disk and virtio as the rescue disk.
        """
        block_device_mapping_v2 = [{
            "boot_index": "0",
            "source_type": "blank",
            "volume_size": CONF.volume.volume_size,
            "destination_type": "volume"}]
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='virtio',
            block_device_mapping_v2=block_device_mapping_v2)
        self._test_stable_device_rescue(server_id, rescue_image_id)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('e4636333-c928-40fc-98b7-70a23eef4224')
    def test_stable_device_rescue_bfv_image_volume(self):
        """Test rescuing server with blank volume as block_device_mapping_v2

        Create a server with block_device_mapping_v2 with image volume,
        then rescue the server with disk and virtio as the rescue disk.
        """
        block_device_mapping_v2 = [{
            "boot_index": "0",
            "source_type": "image",
            "volume_size": CONF.volume.volume_size,
            "uuid": CONF.compute.image_ref,
            "destination_type": "volume"}]
        server_id, rescue_image_id = self._create_server_and_rescue_image(
            hw_rescue_device='disk', hw_rescue_bus='virtio',
            block_device_mapping_v2=block_device_mapping_v2)
        self._test_stable_device_rescue(server_id, rescue_image_id)



# Copyright (c) 2014 Deutsche Telekom AG
# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import abc


from tempest.lib import auth
from tempest.lib import exceptions


class CredentialProvider(object, metaclass=abc.ABCMeta):
    def __init__(self, identity_version, name=None,
                 network_resources=None, credentials_domain=None,
                 admin_role=None, identity_uri=None):
        """A CredentialProvider supplies credentials to test classes.

        :param identity_version: Identity version of the credentials provided
        :param name: Name of the calling test. Included in provisioned
                     credentials when credentials are provisioned on the fly
        :param network_resources: Network resources required for the
                                  credentials
        :param credentials_domain: Domain credentials belong to
        :param admin_role: Name of the role of the admin account
        :param identity_uri: Identity URI of the target cloud. This *must* be
                             specified for anything to work.
        """
        self.identity_version = identity_version
        self.identity_uri = identity_uri
        self.name = name or "test_creds"
        self.network_resources = network_resources
        self.credentials_domain = credentials_domain or 'Default'
        self.admin_role = admin_role
        if not auth.is_identity_version_supported(self.identity_version):
            raise exceptions.InvalidIdentityVersion(
                identity_version=self.identity_version)

    @abc.abstractmethod
    def get_primary_creds(self):
        return

    @abc.abstractmethod
    def get_admin_creds(self):
        return

    @abc.abstractmethod
    def get_alt_creds(self):
        return

    @abc.abstractmethod
    def get_system_admin_creds(self):
        return

    @abc.abstractmethod
    def get_system_member_creds(self):
        return

    @abc.abstractmethod
    def get_system_reader_creds(self):
        return

    @abc.abstractmethod
    def get_domain_admin_creds(self):
        return

    @abc.abstractmethod
    def get_domain_member_creds(self):
        return

    @abc.abstractmethod
    def get_domain_reader_creds(self):
        return

    @abc.abstractmethod
    def get_project_admin_creds(self):
        return

    @abc.abstractmethod
    def get_project_alt_admin_creds(self):
        return

    @abc.abstractmethod
    def get_project_member_creds(self):
        return

    @abc.abstractmethod
    def get_project_alt_member_creds(self):
        return

    @abc.abstractmethod
    def get_project_reader_creds(self):
        return

    @abc.abstractmethod
    def get_project_alt_reader_creds(self):
        return

    @abc.abstractmethod
    def clear_creds(self):
        return

    @abc.abstractmethod
    def is_multi_user(self):
        return

    @abc.abstractmethod
    def is_multi_tenant(self):
        return

    @abc.abstractmethod
    def get_creds_by_roles(self, roles, force_new=False, scope=None):
        return

    @abc.abstractmethod
    def is_role_available(self, role):
        return


class TestResources(object):
    """Readonly Credentials, with network resources added."""

    def __init__(self, credentials):
        self._credentials = credentials
        self.network = None
        self.subnet = None
        self.router = None

    def __getattr__(self, item):
        return getattr(self._credentials, item)

    def __str__(self):
        _format = "Credentials: %s, Network: %s, Subnet: %s, Router: %s"
        return _format % (self._credentials, self.network, self.subnet,
                          self.router)

    def set_resources(self, **kwargs):
        for key in kwargs:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

    @property
    def credentials(self):
        return self._credentials



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.common import utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class EC2CredentialsTest(base.BaseIdentityV2Test):

    @classmethod
    def skip_checks(cls):
        super(EC2CredentialsTest, cls).skip_checks()
        if not utils.is_extension_enabled('OS-EC2', 'identity'):
            msg = "OS-EC2 identity extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(EC2CredentialsTest, cls).resource_setup()
        cls.creds = cls.os_primary.credentials

    @decorators.idempotent_id('b580fab9-7ae9-46e8-8138-417260cb6f9f')
    def test_create_ec2_credential(self):
        """Create user ec2 credential."""
        resp = self.non_admin_users_client.create_user_ec2_credential(
            self.creds.user_id,
            tenant_id=self.creds.tenant_id)["credential"]
        access = resp['access']
        self.addCleanup(
            self.non_admin_users_client.delete_user_ec2_credential,
            self.creds.user_id, access)
        self.assertNotEmpty(resp['access'])
        self.assertNotEmpty(resp['secret'])
        self.assertEqual(self.creds.user_id, resp['user_id'])
        self.assertEqual(self.creds.tenant_id, resp['tenant_id'])

    @decorators.idempotent_id('9e2ea42f-0a4f-468c-a768-51859ce492e0')
    def test_list_ec2_credentials(self):
        """Get the list of user ec2 credentials."""
        created_creds = []
        # create first ec2 credentials
        creds1 = self.non_admin_users_client.create_user_ec2_credential(
            self.creds.user_id,
            tenant_id=self.creds.tenant_id)["credential"]
        created_creds.append(creds1['access'])
        self.addCleanup(
            self.non_admin_users_client.delete_user_ec2_credential,
            self.creds.user_id, creds1['access'])

        # create second ec2 credentials
        creds2 = self.non_admin_users_client.create_user_ec2_credential(
            self.creds.user_id,
            tenant_id=self.creds.tenant_id)["credential"]
        created_creds.append(creds2['access'])
        self.addCleanup(
            self.non_admin_users_client.delete_user_ec2_credential,
            self.creds.user_id, creds2['access'])

        # get the list of user ec2 credentials
        resp = self.non_admin_users_client.list_user_ec2_credentials(
            self.creds.user_id)["credentials"]
        fetched_creds = [cred['access'] for cred in resp]
        # created credentials should be in a fetched list
        missing = [cred for cred in created_creds
                   if cred not in fetched_creds]
        self.assertEmpty(missing,
                         "Failed to find ec2_credentials %s in fetched list" %
                         ', '.join(cred for cred in missing))

    @decorators.idempotent_id('cb284075-b613-440d-83ca-fe0b33b3c2b8')
    def test_show_ec2_credential(self):
        """Get the definite user ec2 credential."""
        resp = self.non_admin_users_client.create_user_ec2_credential(
            self.creds.user_id,
            tenant_id=self.creds.tenant_id)["credential"]
        self.addCleanup(
            self.non_admin_users_client.delete_user_ec2_credential,
            self.creds.user_id, resp['access'])

        ec2_creds = self.non_admin_users_client.show_user_ec2_credential(
            self.creds.user_id, resp['access']
        )["credential"]
        for key in ['access', 'secret', 'user_id', 'tenant_id']:
            self.assertEqual(ec2_creds[key], resp[key])

    @decorators.idempotent_id('6aba0d4c-b76b-4e46-aa42-add79bc1551d')
    def test_delete_ec2_credential(self):
        """Delete user ec2 credential."""
        resp = self.non_admin_users_client.create_user_ec2_credential(
            self.creds.user_id,
            tenant_id=self.creds.tenant_id)["credential"]
        access = resp['access']
        self.non_admin_users_client.delete_user_ec2_credential(
            self.creds.user_id, access)
        self.assertRaises(
            lib_exc.NotFound,
            self.non_admin_users_client.show_user_ec2_credential,
            self.creds.user_id,
            access)



# Copyright 2019 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import access_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestAccessRulesClient(base.BaseServiceTest):
    FAKE_LIST_ACCESS_RULES = {
        "links": {
            "self": "https://example.com/identity/v3/users/" +
                    "3e0716ae/access_rules",
            "previous": None,
            "next": None
        },
        "access_rules": [
            {
                "path": "/v2.0/metrics",
                "links": {
                    "self": "https://example.com/identity/v3/access_rules/" +
                            "07d719df00f349ef8de77d542edf010c"
                },
                "id": "07d719df00f349ef8de77d542edf010c",
                "service": "monitoring",
                "method": "GET"
            }
        ]
    }

    FAKE_ACCESS_RULE_INFO = {
        "access_rule": {
            "path": "/v2.0/metrics",
            "links": {
                "self": "https://example.com/identity/v3/access_rules/" +
                        "07d719df00f349ef8de77d542edf010c"
            },
            "id": "07d719df00f349ef8de77d542edf010c",
            "service": "monitoring",
            "method": "GET"
        }
    }

    def setUp(self):
        super(TestAccessRulesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = access_rules_client.AccessRulesClient(
            fake_auth, 'identity', 'regionOne')

    def _test_show_access_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_access_rule,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ACCESS_RULE_INFO,
            bytes_body,
            user_id="123456",
            access_rule_id="5499a186")

    def _test_list_access_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_access_rules,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ACCESS_RULES,
            bytes_body,
            user_id="123456")

    def test_show_access_rule_with_str_body(self):
        self._test_show_access_rule()

    def test_show_access_rule_with_bytes_body(self):
        self._test_show_access_rule(bytes_body=True)

    def test_list_access_rule_with_str_body(self):
        self._test_list_access_rules()

    def test_list_access_rule_with_bytes_body(self):
        self._test_list_access_rules(bytes_body=True)

    def test_delete_access_rule(self):
        self.check_service_client_function(
            self.client.delete_access_rule,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id="123456",
            access_rule_id="5499a186",
            status=204)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import endpoints_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestEndpointsClient(base.BaseServiceTest):
    FAKE_CREATE_ENDPOINT = {
        "endpoint": {
            "id": 1,
            "tenantId": 1,
            "region": "North",
            "type": "compute",
            "publicURL": "https://compute.north.public.com/v1",
            "internalURL": "https://compute.north.internal.com/v1",
            "adminURL": "https://compute.north.internal.com/v1"
        }
    }

    FAKE_LIST_ENDPOINTS = {
        "endpoints": [
            {
                "id": 1,
                "tenantId": "1",
                "region": "North",
                "type": "compute",
                "publicURL": "https://compute.north.public.com/v1",
                "internalURL": "https://compute.north.internal.com/v1",
                "adminURL": "https://compute.north.internal.com/v1"
            },
            {
                "id": 2,
                "tenantId": "1",
                "region": "South",
                "type": "compute",
                "publicURL": "https://compute.north.public.com/v1",
                "internalURL": "https://compute.north.internal.com/v1",
                "adminURL": "https://compute.north.internal.com/v1"
            }
        ]
    }

    FAKE_SERVICE_ID = "a4dc5060-f757-4662-b658-edd2aefbb41d"
    FAKE_ENDPOINT_ID = "b335d394-cdb9-4519-b95d-160b7706e54ew"

    FAKE_UPDATE_ENDPOINT = {
        "endpoint": {
            "id": "828384",
            "interface": "internal",
            "links": {
                "self": "http://example.com/identity/v3/"
                        "endpoints/828384"
            },
            "region_id": "north",
            "service_id": "686766",
            "url": "http://example.com/identity/v3/"
                   "endpoints/828384"
        }
    }

    FAKE_SHOW_ENDPOINT = {
        "endpoint": {
            "enabled": True,
            "id": "01c3d5b92f7841ac83fb4b26173c12c7",
            "interface": "admin",
            "links": {
                "self": "http://example.com/identity/v3/"
                        "endpoints/828384"
            },
            "region": "RegionOne",
            "region_id": "RegionOne",
            "service_id": "3b2d6ad7e02c4cde8498a547601f1b8f",
            "url": "http://23.253.211.234:9696/"
        }
    }

    def setUp(self):
        super(TestEndpointsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = endpoints_client.EndPointsClient(
            fake_auth, 'identity', 'regionOne')

    def _test_create_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_endpoint,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_ENDPOINT,
            bytes_body,
            status=201,
            service_id="b344506af7644f6794d9cb316600b020",
            region="region-demo",
            publicurl="https://compute.north.public.com/v1",
            adminurl="https://compute.north.internal.com/v1",
            internalurl="https://compute.north.internal.com/v1")

    def _test_list_endpoints(self, bytes_body=False, mock_args='endpoints',
                             **params):
        self.check_service_client_function(
            self.client.list_endpoints,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINTS,
            bytes_body,
            mock_args=[mock_args],
            **params)

    def _test_update_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_endpoint,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_UPDATE_ENDPOINT,
            bytes_body,
            endpoint_id=self.FAKE_ENDPOINT_ID,
            interface="public",
            region_id="north",
            url="http://example.com/identity/v3/endpoints/828384",
            service_id=self.FAKE_SERVICE_ID)

    def _test_show_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_endpoint,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_ENDPOINT,
            bytes_body,
            endpoint_id="3456")

    def test_update_endpoint_with_str_body(self):
        self._test_update_endpoint()

    def test_update_endpoint_with_bytes_body(self):
        self._test_update_endpoint(bytes_body=True)

    def test_show_endpoint_with_str_body(self):
        self._test_show_endpoint()

    def test_show_endpoint_with_bytes_body(self):
        self._test_show_endpoint(bytes_body=True)

    def test_create_endpoint_with_str_body(self):
        self._test_create_endpoint()

    def test_create_endpoint_with_bytes_body(self):
        self._test_create_endpoint(bytes_body=True)

    def test_list_endpoints_with_str_body(self):
        self._test_list_endpoints()

    def test_list_endpoints_with_bytes_body(self):
        self._test_list_endpoints(bytes_body=True)

    def test_list_endpoints_with_params(self):
        # Run the test separately for each param, to avoid assertion error
        # resulting from randomized params order.
        mock_args = 'endpoints?service_id=%s' % self.FAKE_SERVICE_ID
        self._test_list_endpoints(mock_args=mock_args,
                                  service_id=self.FAKE_SERVICE_ID)

        mock_args = 'endpoints?interface=public'
        self._test_list_endpoints(mock_args=mock_args, interface='public')

    def test_delete_endpoint(self):
        self.check_service_client_function(
            self.client.delete_endpoint,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            endpoint_id="b344506af7644f6794d9cb316600b020",
            status=204)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.object_storage import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class ContainerNegativeTest(base.BaseObjectTest):
    """Negative tests of containers"""

    @classmethod
    def resource_setup(cls):
        super(ContainerNegativeTest, cls).resource_setup()

        if CONF.object_storage_feature_enabled.discoverability:
            # use /info to get default constraints
            body = cls.capabilities_client.list_capabilities()
            cls.constraints = body['swift']

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('30686921-4bed-4764-a038-40d741ed4e78')
    @testtools.skipUnless(
        CONF.object_storage_feature_enabled.discoverability,
        'Discoverability function is disabled')
    def test_create_container_name_exceeds_max_length(self):
        """Test creating container with name longer than max"""
        max_length = self.constraints['max_container_name_length']
        # create a container with long name
        container_name = data_utils.arbitrary_string(size=max_length + 1)
        ex = self.assertRaises(
            exceptions.BadRequest,
            self.container_client.update_container,
            container_name)
        self.assertIn('Container name length of ' + str(max_length + 1) +
                      ' longer than ' + str(max_length), str(ex))

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('41e645bf-2e68-4f84-bf7b-c71aa5cd76ce')
    @testtools.skipUnless(
        CONF.object_storage_feature_enabled.discoverability,
        'Discoverability function is disabled')
    def test_create_container_metadata_name_exceeds_max_length(self):
        """Test creating container with metadata name longer than max"""
        max_length = self.constraints['max_meta_name_length']
        container_name = data_utils.rand_name(name='TestContainer')
        metadata_name = 'X-Container-Meta-' + data_utils.arbitrary_string(
            size=max_length + 1)
        metadata = {metadata_name: 'penguin'}
        ex = self.assertRaises(
            exceptions.BadRequest,
            self.container_client.update_container,
            container_name, **metadata)
        self.assertIn('Metadata name too long', str(ex))

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('81e36922-326b-4b7c-8155-3bbceecd7a82')
    @testtools.skipUnless(
        CONF.object_storage_feature_enabled.discoverability,
        'Discoverability function is disabled')
    def test_create_container_metadata_value_exceeds_max_length(self):
        """Test creating container with metadata value longer than max"""
        max_length = self.constraints['max_meta_value_length']
        container_name = data_utils.rand_name(name='TestContainer')
        metadata_value = data_utils.arbitrary_string(size=max_length + 1)
        metadata = {'X-Container-Meta-animal': metadata_value}
        ex = self.assertRaises(
            exceptions.BadRequest,
            self.container_client.update_container,
            container_name, **metadata)
        self.assertIn('Metadata value longer than ' + str(max_length), str(ex))

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('ac666539-d566-4f02-8ceb-58e968dfb732')
    @testtools.skipUnless(
        CONF.object_storage_feature_enabled.discoverability,
        'Discoverability function is disabled')
    def test_create_container_metadata_exceeds_overall_metadata_count(self):
        """Test creating container with metadata exceeding default count"""
        max_count = self.constraints['max_meta_count']
        container_name = data_utils.rand_name(name='TestContainer')
        metadata = {}
        for i in range(max_count + 1):
            metadata['X-Container-Meta-animal-' + str(i)] = 'penguin'

        ex = self.assertRaises(
            exceptions.BadRequest,
            self.container_client.update_container,
            container_name, **metadata)
        self.assertIn('Too many metadata items; max ' + str(max_count),
                      str(ex))

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('1a95ab2e-b712-4a98-8a4d-8ce21b7557d6')
    def test_get_metadata_headers_with_invalid_container_name(self):
        """Test getting metadata headers with invalid container name"""
        self.assertRaises(exceptions.NotFound,
                          self.container_client.list_container_metadata,
                          'invalid_container_name')

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('125a24fa-90a7-4cfc-b604-44e49d788390')
    def test_update_metadata_with_nonexistent_container_name(self):
        """Test updating metadata using a nonexistent container name"""
        metadata = {'animal': 'penguin'}

        self.assertRaises(
            exceptions.NotFound,
            self.container_client.create_update_or_delete_container_metadata,
            'nonexistent_container_name', create_update_metadata=metadata)

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('65387dbf-a0e2-4aac-9ddc-16eb3f1f69ba')
    def test_delete_with_nonexistent_container_name(self):
        """Test deleting metadata using a non existent container name"""
        metadata = {'animal': 'penguin'}

        self.assertRaises(
            exceptions.NotFound,
            self.container_client.create_update_or_delete_container_metadata,
            'nonexistent_container_name', delete_metadata=metadata)

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('14331d21-1e81-420a-beea-19cb5e5207f5')
    def test_list_all_container_objects_with_nonexistent_container(self):
        """Test getting a list of all objects on a non existent container"""
        params = {'limit': 9999, 'format': 'json'}
        self.assertRaises(exceptions.NotFound,
                          self.container_client.list_container_objects,
                          'nonexistent_container_name', params)

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('86b2ab08-92d5-493d-acd2-85f0c848819e')
    def test_list_all_container_objects_on_deleted_container(self):
        """Test getting a list of all objects on a deleted container"""
        container_name = self.create_container()
        # delete container
        resp, _ = self.container_client.delete_container(container_name)
        self.assertHeaders(resp, 'Container', 'DELETE')
        params = {'limit': 9999, 'format': 'json'}
        self.assertRaises(exceptions.NotFound,
                          self.container_client.list_container_objects,
                          container_name, params)

    @decorators.attr(type=["negative"])
    @decorators.idempotent_id('42da116e-1e8c-4c96-9e06-2f13884ed2b1')
    def test_delete_non_empty_container(self):
        """Test deleting a container with object in it"""
        # create a container and an object within it
        # attempt to delete a container that isn't empty.
        container_name = self.create_container()
        self.addCleanup(self.container_client.delete_container,
                        container_name)
        object_name, _ = self.create_object(container_name)
        self.addCleanup(self.object_client.delete_object,
                        container_name, object_name)

        ex = self.assertRaises(exceptions.Conflict,
                               self.container_client.delete_container,
                               container_name)
        self.assertIn('There was a conflict when trying to complete your '
                      'request.', str(ex))



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import operator

from oslo_utils import timeutils
import six

from tempest.api.identity import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class TokensV3Test(base.BaseIdentityV3Test):
    """Test identity tokens"""

    @decorators.idempotent_id('a9512ac3-3909-48a4-b395-11f438e16260')
    def test_validate_token(self):
        """Test validating token for user"""
        creds = self.os_primary.credentials
        user_id = creds.user_id
        username = creds.username
        password = creds.password
        user_domain_id = creds.user_domain_id
        # GET and validate token
        subject_token, token_body = self.non_admin_token.get_token(
            user_id=user_id,
            username=username,
            user_domain_id=user_domain_id,
            password=password,
            auth_data=True)
        authenticated_token = self.non_admin_client.show_token(
            subject_token)['token']
        # sanity checking to make sure they are indeed the same token
        # If there are roles in the token, sort the roles
        authenticated_token_roles = authenticated_token.get("roles")
        if authenticated_token_roles:
            authenticated_token["roles"] = authenticated_token_roles.sort(
                key=operator.itemgetter('id'))
        token_body_roles = token_body.get("roles")
        if token_body_roles:
            token_body["roles"] = token_body_roles.sort(
                key=operator.itemgetter('id'))
        self.assertEqual(authenticated_token, token_body)
        # test to see if token has been properly authenticated
        self.assertEqual(authenticated_token['user']['id'], user_id)
        # NOTE: resource name that are case-sensitive in keystone
        # depends on backends such as MySQL or LDAP which are
        # case-insensitive, case-preserving. Resource name is
        # returned as it is stored in the backend, not as it is
        # requested. Verifying the username with both lower-case to
        # avoid failure on different backends
        self.assertEqual(
            authenticated_token['user']['name'].lower(), username.lower())

        self.non_admin_client.delete_token(subject_token)
        self.assertRaises(
            lib_exc.NotFound, self.non_admin_client.show_token, subject_token)

    @decorators.idempotent_id('6f8e4436-fc96-4282-8122-e41df57197a9')
    def test_create_token(self):
        """Test creating token for user"""
        creds = self.os_primary.credentials
        user_id = creds.user_id
        username = creds.username
        password = creds.password
        user_domain_id = creds.user_domain_id

        # 'user_domain_id' needs to be specified otherwise tempest.lib assumes
        # it to be 'default'
        token_id, resp = self.non_admin_token.get_token(
            user_id=user_id,
            username=username,
            user_domain_id=user_domain_id,
            password=password,
            auth_data=True)

        self.assertNotEmpty(token_id)
        self.assertIsInstance(token_id, six.string_types)

        now = timeutils.utcnow()
        expires_at = timeutils.normalize_time(
            timeutils.parse_isotime(resp['expires_at']))
        self.assertGreater(resp['expires_at'],
                           resp['issued_at'])
        self.assertGreater(expires_at, now)

        subject_id = resp['user']['id']
        if user_id:
            self.assertEqual(subject_id, user_id)
        else:
            # Expect a user ID, but don't know what it will be.
            self.assertIsNotNone(subject_id, 'Expected user ID in token.')

        subject_name = resp['user']['name']

        if username:
            # NOTE: resource name that are case-sensitive in keystone
            # depends on backends such as MySQL or LDAP which are
            # case-insensitive, case-preserving. Resource name is
            # returned as it is stored in the backend, not as it is
            # requested. Verifying the username with both lower-case to
            # avoid failure on different backends
            self.assertEqual(subject_name.lower(), username.lower())
        else:
            # Expect a user name, but don't know what it will be
            self.assertIsNotNone(subject_name, 'Expected user name in token.')

        self.assertEqual(resp['methods'][0], 'password')

    @decorators.idempotent_id('0f9f5a5f-d5cd-4a86-8a5b-c5ded151f212')
    def test_token_auth_creation_existence_deletion(self):
        """Test auth/check existence/delete token for user

        Tests basic token auth functionality in a way that is compatible with
        pre-provisioned credentials. The default user is used for token
        authentication.
        """

        # Valid user's token is authenticated
        user = self.os_primary.credentials
        # Perform Authentication
        resp = self.non_admin_token.auth(
            user_id=user.user_id, password=user.password).response
        subject_token = resp['x-subject-token']
        self.non_admin_client.check_token_existence(subject_token)
        # Perform GET Token
        token_details = self.non_admin_client.show_token(
            subject_token)['token']
        self.assertEqual(resp['x-subject-token'], subject_token)
        self.assertEqual(token_details['user']['id'], user.user_id)
        # NOTE: resource name that are case-sensitive in keystone
        # depends on backends such as MySQL or LDAP which are
        # case-insensitive, case-preserving. Resource name is
        # returned as it is stored in the backend, not as it is
        # requested. Verifying the username with both lower-case to
        # avoid failure on different backends
        self.assertEqual(
            token_details['user']['name'].lower(),
            user.username.lower())
        # Perform Delete Token
        self.non_admin_client.delete_token(subject_token)
        self.assertRaises(lib_exc.NotFound,
                          self.non_admin_client.check_token_existence,
                          subject_token)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions


CONF = config.CONF


class IdentityV3UsersTest(base.BaseIdentityV3Test):
    """Test identity user password"""

    @classmethod
    def resource_setup(cls):
        super(IdentityV3UsersTest, cls).resource_setup()
        cls.creds = cls.os_primary.credentials
        cls.user_id = cls.creds.user_id

    def _update_password(self, original_password, password):
        self.non_admin_users_client.update_user_password(
            self.user_id,
            password=password,
            original_password=original_password)

        # NOTE(morganfainberg): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure
        # we are passing the second boundary.
        time.sleep(1)

        # check authorization with new password
        self.non_admin_token.auth(user_id=self.user_id, password=password)

        # Reset auth to get a new token with the new password
        self.non_admin_users_client.auth_provider.clear_auth()
        self.non_admin_users_client.auth_provider.credentials.password = (
            password)

    def _restore_password(self, old_pass, new_pass):
        if CONF.identity_feature_enabled.security_compliance:
            # First we need to clear the password history
            unique_count = CONF.identity.user_unique_last_password_count
            for _ in range(unique_count):
                random_pass = data_utils.rand_password()
                self._update_password(
                    original_password=new_pass, password=random_pass)
                new_pass = random_pass

        self._update_password(original_password=new_pass, password=old_pass)
        # Reset auth again to verify the password restore does work.
        # Clear auth restores the original credentials and deletes
        # cached auth data
        self.non_admin_users_client.auth_provider.clear_auth()
        # NOTE(lbragstad): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure we
        # are passing the second boundary before attempting to
        # authenticate.
        time.sleep(1)
        self.non_admin_users_client.auth_provider.set_auth()

    @decorators.idempotent_id('ad71bd23-12ad-426b-bb8b-195d2b635f27')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_user_update_own_password(self):
        """Test updating user's own password"""
        old_pass = self.creds.password
        old_token = self.non_admin_client.token
        new_pass = data_utils.rand_password()

        # to change password back. important for use_dynamic_credentials=false
        self.addCleanup(self._restore_password, old_pass, new_pass)

        # user updates own password
        self._update_password(original_password=old_pass, password=new_pass)

        # authorize with old token should lead to IdentityError (404 code)
        self.assertRaises(exceptions.IdentityError,
                          self.non_admin_token.auth,
                          token=old_token)

        # authorize with old password should lead to Unauthorized
        self.assertRaises(exceptions.Unauthorized,
                          self.non_admin_token.auth,
                          user_id=self.user_id,
                          password=old_pass)

    @testtools.skipUnless(CONF.identity_feature_enabled.security_compliance,
                          'Security compliance not available.')
    @decorators.idempotent_id('941784ee-5342-4571-959b-b80dd2cea516')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_password_history_check_self_service_api(self):
        """Test checking password changing history"""
        old_pass = self.creds.password
        new_pass1 = data_utils.rand_password()
        new_pass2 = data_utils.rand_password()

        self.addCleanup(self._restore_password, old_pass, new_pass2)

        # Update password
        self._update_password(original_password=old_pass, password=new_pass1)

        if CONF.identity.user_unique_last_password_count > 1:
            # Can not reuse a previously set password
            self.assertRaises(exceptions.BadRequest,
                              self.non_admin_users_client.update_user_password,
                              self.user_id,
                              password=new_pass1,
                              original_password=new_pass1)

            self.assertRaises(exceptions.BadRequest,
                              self.non_admin_users_client.update_user_password,
                              self.user_id,
                              password=old_pass,
                              original_password=new_pass1)

        # A different password can be set
        self._update_password(original_password=new_pass1, password=new_pass2)

    @testtools.skipUnless(CONF.identity_feature_enabled.security_compliance,
                          'Security compliance not available.')
    @decorators.idempotent_id('a7ad8bbf-2cff-4520-8c1d-96332e151658')
    def test_user_account_lockout(self):
        """Test locking out user account after failure attempts"""
        if (CONF.identity.user_lockout_failure_attempts <= 0 or
                CONF.identity.user_lockout_duration <= 0):
            raise self.skipException(
                "Both CONF.identity.user_lockout_failure_attempts and "
                "CONF.identity.user_lockout_duration should be greater than "
                "zero to test this feature")

        password = self.creds.password

        # First, we login using the correct credentials
        self.non_admin_token.auth(user_id=self.user_id, password=password)

        # Lock user account by using the wrong password to login
        bad_password = data_utils.rand_password()
        for _ in range(CONF.identity.user_lockout_failure_attempts):
            self.assertRaises(exceptions.Unauthorized,
                              self.non_admin_token.auth,
                              user_id=self.user_id,
                              password=bad_password)

        # The user account must be locked, so now it is not possible to login
        # even using the correct password
        self.assertRaises(exceptions.Unauthorized,
                          self.non_admin_token.auth,
                          user_id=self.user_id,
                          password=password)

        # If we wait the required time, the user account will be unlocked
        time.sleep(CONF.identity.user_lockout_duration + 1)
        self.non_admin_token.auth(user_id=self.user_id, password=password)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common import fixed_network
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


CONF = config.CONF


class ListServerFiltersTestJSON(base.BaseV2ComputeTest):
    """Test listing servers filtered by specified attribute"""

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, dhcp=True)
        super(ListServerFiltersTestJSON, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ListServerFiltersTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ListServerFiltersTestJSON, cls).resource_setup()

        network = cls.get_tenant_network()
        if network:
            cls.fixed_network_name = network.get('name')
        else:
            cls.fixed_network_name = None
        network_kwargs = fixed_network.set_networks_kwarg(network)
        cls.s1_name = data_utils.rand_name(cls.__name__ + '-instance')
        cls.s1 = cls.create_test_server(name=cls.s1_name, **network_kwargs)

        cls.s2_name = data_utils.rand_name(cls.__name__ + '-instance')
        # If image_ref_alt is "" or None then we still want to boot a server
        # but we rely on `testtools.skipUnless` decorator to actually skip
        # the irrelevant tests.
        cls.s2 = cls.create_test_server(
            name=cls.s2_name, image_id=cls.image_ref_alt or cls.image_ref)

        cls.s3_name = data_utils.rand_name(cls.__name__ + '-instance')
        cls.s3 = cls.create_test_server(name=cls.s3_name,
                                        flavor=cls.flavor_ref_alt,
                                        wait_until='ACTIVE')

        waiters.wait_for_server_status(cls.client, cls.s1['id'],
                                       'ACTIVE')
        waiters.wait_for_server_status(cls.client, cls.s2['id'],
                                       'ACTIVE')

    @decorators.idempotent_id('05e8a8e7-9659-459a-989d-92c2f501f4ba')
    @testtools.skipUnless(CONF.compute.image_ref != CONF.compute.image_ref_alt,
                          "Need distinct images to run this test")
    def test_list_servers_filter_by_image(self):
        """Filter the list of servers by image"""
        params = {'image': self.image_ref}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('573637f5-7325-47bb-9144-3476d0416908')
    def test_list_servers_filter_by_flavor(self):
        """Filter the list of servers by flavor"""
        params = {'flavor': self.flavor_ref_alt}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertNotIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('9b067a7b-7fee-4f6a-b29c-be43fe18fc5a')
    def test_list_servers_filter_by_server_name(self):
        """Filter the list of servers by server name"""
        params = {'name': self.s1_name}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s2_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s3_name, map(lambda x: x['name'], servers))

    @decorators.idempotent_id('ca78e20e-fddb-4ce6-b7f7-bcbf8605e66e')
    def test_list_servers_filter_by_active_status(self):
        """Filter the list of servers by server active status"""
        params = {'status': 'active'}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('451dbbb2-f330-4a9f-b0e1-5f5d2cb0f34c')
    def test_list_servers_filter_by_shutoff_status(self):
        """Filter the list of servers by server shutoff status"""
        params = {'status': 'shutoff'}
        self.client.stop_server(self.s1['id'])
        waiters.wait_for_server_status(self.client, self.s1['id'],
                                       'SHUTOFF')
        body = self.client.list_servers(**params)
        self.client.start_server(self.s1['id'])
        waiters.wait_for_server_status(self.client, self.s1['id'],
                                       'ACTIVE')
        servers = body['servers']

        self.assertIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('614cdfc1-d557-4bac-915b-3e67b48eee76')
    def test_list_servers_filter_by_limit(self):
        """Filter the list of servers by limit 1

        Verify only the expected number of servers are returned (one server)
        """
        params = {'limit': 1}
        servers = self.client.list_servers(**params)
        self.assertEqual(1, len([x for x in servers['servers'] if 'id' in x]))

    @decorators.idempotent_id('b1495414-2d93-414c-8019-849afe8d319e')
    def test_list_servers_filter_by_zero_limit(self):
        """Filter the list of servers by limit 0

        Verify only the expected number of servers are returned (no server)
        """
        params = {'limit': 0}
        servers = self.client.list_servers(**params)
        self.assertEmpty(servers['servers'])

    @decorators.idempotent_id('37791bbd-90c0-4de0-831e-5f38cba9c6b3')
    def test_list_servers_filter_by_exceed_limit(self):
        """Filter the list of servers by exceeded limit

        Verify only the expected number of servers are returned (all servers)
        """
        params = {'limit': 100000}
        servers = self.client.list_servers(**params)
        all_servers = self.client.list_servers()
        self.assertEqual(len([x for x in all_servers['servers'] if 'id' in x]),
                         len([x for x in servers['servers'] if 'id' in x]))

    @decorators.idempotent_id('b3304c3b-97df-46d2-8cd3-e2b6659724e7')
    @testtools.skipUnless(CONF.compute.image_ref != CONF.compute.image_ref_alt,
                          "Need distinct images to run this test")
    def test_list_servers_detailed_filter_by_image(self):
        """"Filter the detailed list of servers by image"""
        params = {'image': self.image_ref}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']

        self.assertIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('80c574cc-0925-44ba-8602-299028357dd9')
    def test_list_servers_detailed_filter_by_flavor(self):
        """Filter the detailed list of servers by flavor"""
        params = {'flavor': self.flavor_ref_alt}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']

        self.assertNotIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('f9eb2b70-735f-416c-b260-9914ac6181e4')
    def test_list_servers_detailed_filter_by_server_name(self):
        """Filter the detailed list of servers by server name"""
        params = {'name': self.s1_name}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s2_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s3_name, map(lambda x: x['name'], servers))

    @decorators.idempotent_id('de2612ab-b7dd-4044-b0b1-d2539601911f')
    def test_list_servers_detailed_filter_by_server_status(self):
        """Filter the detailed list of servers by server status"""
        params = {'status': 'active'}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']
        test_ids = [s['id'] for s in (self.s1, self.s2, self.s3)]

        self.assertIn(self.s1['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s2['id'], map(lambda x: x['id'], servers))
        self.assertIn(self.s3['id'], map(lambda x: x['id'], servers))
        self.assertEqual(['ACTIVE'] * 3, [x['status'] for x in servers
                                          if x['id'] in test_ids])

    @decorators.idempotent_id('e9f624ee-92af-4562-8bec-437945a18dcb')
    def test_list_servers_filtered_by_name_wildcard(self):
        """Filter the list of servers by part of server name"""
        # List all servers that contains '-instance' in name
        params = {'name': '-instance'}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
        self.assertIn(self.s2_name, map(lambda x: x['name'], servers))
        self.assertIn(self.s3_name, map(lambda x: x['name'], servers))

        # Let's take random part of name and try to search it
        part_name = self.s1_name[6:-1]

        params = {'name': part_name}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s2_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s3_name, map(lambda x: x['name'], servers))

    @decorators.idempotent_id('24a89b0c-0d55-4a28-847f-45075f19b27b')
    def test_list_servers_filtered_by_name_regex(self):
        """Filter the list of servers by server name regular expression"""
        # list of regex that should match s1, s2 and s3
        regexes = [r'^.*\-instance\-[0-9]+$', r'^.*\-instance\-.*$']
        for regex in regexes:
            params = {'name': regex}
            body = self.client.list_servers(**params)
            servers = body['servers']

            self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
            self.assertIn(self.s2_name, map(lambda x: x['name'], servers))
            self.assertIn(self.s3_name, map(lambda x: x['name'], servers))

        # Let's take random part of name and try to search it
        part_name = self.s1_name[-10:]

        params = {'name': part_name}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s2_name, map(lambda x: x['name'], servers))
        self.assertNotIn(self.s3_name, map(lambda x: x['name'], servers))

    @decorators.idempotent_id('43a1242e-7b31-48d1-88f2-3f72aa9f2077')
    def test_list_servers_filtered_by_ip(self):
        """Filter the list of servers by server ip address"""
        # Here should be listed 1 server
        if not self.fixed_network_name:
            msg = 'fixed_network_name needs to be configured to run this test'
            raise self.skipException(msg)

        # list servers filter by ip is something "regexp match", i.e,
        # filter by "10.1.1.1" will return both "10.1.1.1" and "10.1.1.10".
        # so here look for the longest server ip, and filter by that ip,
        # so as to ensure only one server is returned.
        ip_list = {}
        self.s1 = self.client.show_server(self.s1['id'])['server']
        # Get first ip address in spite of v4 or v6
        ip_addr = self.s1['addresses'][self.fixed_network_name][0]['addr']
        ip_list[ip_addr] = self.s1['id']

        self.s2 = self.client.show_server(self.s2['id'])['server']
        ip_addr = self.s2['addresses'][self.fixed_network_name][0]['addr']
        ip_list[ip_addr] = self.s2['id']

        self.s3 = self.client.show_server(self.s3['id'])['server']
        ip_addr = self.s3['addresses'][self.fixed_network_name][0]['addr']
        ip_list[ip_addr] = self.s3['id']

        longest_ip = max([[len(ip), ip] for ip in ip_list])[1]
        params = {'ip': longest_ip}
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(ip_list[longest_ip], map(lambda x: x['id'], servers))
        del ip_list[longest_ip]
        for ip in ip_list:
            self.assertNotIn(ip_list[ip], map(lambda x: x['id'], servers))

    @decorators.idempotent_id('a905e287-c35e-42f2-b132-d02b09f3654a')
    def test_list_servers_filtered_by_ip_regex(self):
        """Filter the list of servers by part of server ip address"""
        if not self.fixed_network_name:
            msg = 'fixed_network_name needs to be configured to run this test'
            raise self.skipException(msg)
        # query addresses of the 3 servers
        addrs = []
        for s in [self.s1, self.s2, self.s3]:
            s_show = self.client.show_server(s['id'])['server']
            addr_spec = s_show['addresses'][self.fixed_network_name][0]
            addrs.append(addr_spec['addr'])
        # find common part of the 3 ip addresses
        prefix = ''
        addrs_len = [len(a) for a in addrs]
        addrs_len.sort()
        # iterate over the smallest length of an ip
        for i in range(addrs_len[0]):
            if not addrs[0][i] == addrs[1][i] == addrs[2][i]:
                break
            prefix += addrs[0][i]

        if addr_spec['version'] == 4:
            params = {'ip': prefix}
        else:
            params = {'ip6': prefix}
        # capture all servers in case something goes wrong
        all_servers = self.client.list_servers(detail=True)
        body = self.client.list_servers(**params)
        servers = body['servers']

        self.assertIn(self.s1_name, map(lambda x: x['name'], servers),
                      "%s not found in %s, all servers %s" %
                      (self.s1_name, servers, all_servers))
        self.assertIn(self.s2_name, map(lambda x: x['name'], servers),
                      "%s not found in %s, all servers %s" %
                      (self.s2_name, servers, all_servers))
        self.assertIn(self.s3_name, map(lambda x: x['name'], servers),
                      "%s not found in %s, all servers %s" %
                      (self.s3_name, servers, all_servers))

    @decorators.idempotent_id('67aec2d0-35fe-4503-9f92-f13272b867ed')
    def test_list_servers_detailed_limit_results(self):
        """Filter the detailed list of servers by limit 1

        Verify only the expected number of servers are returned (one server)
        """
        params = {'limit': 1}
        servers = self.client.list_servers(detail=True, **params)
        self.assertEqual(1, len(servers['servers']))
